describe('ProfileCtrl: ', function() {
  // Load the module with Controller
  beforeEach(module('cmb'));

  var ctrl, scope;

  beforeEach(inject(function($controller, $rootScope) {
    // Create a new scope that's a child of the $rootScope
    scope = $rootScope.$new();
    // Create the controller
    ctrl = $controller('ProfileCtrl', {
      $scope: scope
    });
  }));

  it('trying to access profile if not logged in should redirect to login',
    inject(function($state, $auth, $controller, $location){
      spyOn($location, 'path').and.callFake(function(params){});
      spyOn($auth, 'isAuthenticated').and.callFake(function() {
        return false;
      });
      ctrl = $controller('ProfileCtrl', {
        $scope: scope
      });
      $state.go('profile');
      expect($location.path).toHaveBeenCalledWith('/login');
    }));

  it('trying to access profile of not logged in should redirect to login',
    inject(function($state, $auth, $controller, $location){
      spyOn($location, 'path').and.callFake(function(params){});
      spyOn($auth, 'isAuthenticated').and.callFake(function() {
        return true;
      });
      ctrl = $controller('ProfileCtrl', {
        $scope: scope
      });
      $state.go('profile');
    }));

  describe('getProfile http get request: ', function() {
        beforeEach(inject(function($httpBackend) {
          httpBackend = $httpBackend;
          httpBackend.whenGET(/^.*$/).respond(500, '');
        }));

        afterEach(function() {
          httpBackend.verifyNoOutstandingExpectation();
          httpBackend.verifyNoOutstandingRequest();
        });

        it('success', function(){
          var response = JSON.stringify({'user':{'name':'username', 'email':'user@email.com'}});
          httpBackend.expectGET(scope.apiurl+'/me', function(headers){
            return !_.isUndefined(headers.Authorization);
          }).respond(200, response);
          scope.getProfile();
          httpBackend.flush();
          expect(scope.user.name).toEqual('username');
          expect(scope.user.email).toEqual('user@email.com');
        });

        it('failed', function(){
          var response = JSON.stringify({'message':'error msg'});
          httpBackend.expectGET(scope.apiurl+'/me')
              .respond(500, response);
          scope.getProfile();
          httpBackend.flush();
          expect(scope.get_profile_alert.$options.content).toEqual('Could not connect to server!');
        });
      });

  describe('updatePassword: ', function() {
        beforeEach(inject(function($httpBackend) {
          httpBackend = $httpBackend;
          httpBackend.whenGET(/^.*$/).respond(500, '');
        }));

        afterEach(function() {
          httpBackend.verifyNoOutstandingExpectation();
          httpBackend.verifyNoOutstandingRequest();
        });

        it('should check if new and confirm new password is equal', function(){
          scope.newPassword = "pw";
          scope.confirmNewPassword = "not_pw";
          scope.updatePassword();
          expect(scope.update_password_alert.$options.content).toEqual('New Passwords did not match!');

          expect(scope.newPassword).toEqual("");
          expect(scope.confirmNewPassword).toEqual("");
          expect(scope.oldPassword).toEqual("");
          httpBackend.flush();
        });

        it('successfull request', function(){
          scope.myForm = {
              $valid: true,
              $setPristine: function() {}
          };
          spyOn(scope.myForm, '$setPristine').and.callFake(function() {});

          scope.oldPassword = "old_pw";
          scope.newPassword = "pw";
          scope.confirmNewPassword = "pw";

          var profileData = {
            type: 'password',
            password: btoa("old_pw"),
            new_password: btoa("pw")
          };
          var response = JSON.stringify({'user':{'name':'username', 'email':'user@email.com'}});
          httpBackend.expectPUT(scope.apiurl+'/me', profileData, function(headers){
            return !_.isUndefined(headers.Authorization);
          }).respond(200, response);

          scope.updatePassword();
          httpBackend.flush();
          expect(scope.user.name).toEqual('username');
          expect(scope.user.email).toEqual('user@email.com');
          expect(scope.updating_password_alert.$options.content).toEqual('Updating Password');
          expect(scope.updated_password_alert.$options.content).toEqual('Password Updated');

          expect(scope.myForm.$setPristine).toHaveBeenCalled();
          expect(scope.newPassword).toEqual("");
          expect(scope.confirmNewPassword).toEqual("");
          expect(scope.oldPassword).toEqual("");
        });

        it('failed request', function(){
          scope.oldPassword = "old_pw";
          scope.newPassword = "pw";
          scope.confirmNewPassword = "pw";
          scope.myForm = {
              oldPassword: {},
              $valid: true,
              $setPristine: function() {}
          };

          httpBackend.expectPUT(scope.apiurl+'/me').respond(500, "error");
          scope.updatePassword();
          httpBackend.flush();
          expect(scope.update_password_alert.$options.content).toEqual('Could not connect to server!');

          expect(scope.newPassword).toEqual("");
          expect(scope.confirmNewPassword).toEqual("");
          expect(scope.oldPassword).toEqual("");
        });
      });

  describe('updateEmail: ', function() {
        beforeEach(inject(function($httpBackend) {
          httpBackend = $httpBackend;
          httpBackend.whenGET(/^.*$/).respond(500, '');
        }));

        afterEach(function() {
          httpBackend.verifyNoOutstandingExpectation();
          httpBackend.verifyNoOutstandingRequest();
        });

        it('should check if new and confirmed new email is equal', function(){
          scope.newEmail = "email";
          scope.confirmEmail = "not_email";
          scope.emailForm = {
              newEmail: {},
              confirmEmail: {},
              $valid: true,
              $setPristine: function() {}
          };
          spyOn(scope.emailForm, '$setPristine').and.callFake(function() {});
          scope.updateEmail();
          httpBackend.flush();

          expect(scope.update_email_alert.$options.content).toEqual('Emails did not match.');
          expect(scope.newEmail).toEqual("");
          expect(scope.confirmEmail).toEqual("");
          expect(scope.password).toEqual("");
        });

        it('successfull request', function(){
          scope.newEmail = "email";
          scope.confirmEmail = "email";
          scope.password = "pw";
          scope.emailForm = {
              newEmail: {},
              confirmEmail: {},
              $valid: true,
              $setPristine: function() {}
          };
          spyOn(scope.emailForm, '$setPristine').and.callFake(function() {});
          var profileData = {
            type: 'email',
            password: btoa("pw"),
            email: "email"
          };
          var response = JSON.stringify({'user':{'name':'username', 'email':'user@email.com'}});
          httpBackend.expectPUT(scope.apiurl+'/me', profileData, function(headers){
            return !_.isUndefined(headers.Authorization);
          }).respond(200, response);

          scope.updateEmail();
          httpBackend.flush();

          expect(scope.user.name).toEqual('username');
          expect(scope.user.email).toEqual('user@email.com');
          expect(scope.update_email_alert.$options.content).toEqual('Email Updated');

          expect(scope.emailForm.$setPristine).toHaveBeenCalled();
          expect(scope.newEmail).toEqual("");
          expect(scope.confirmEmail).toEqual("");
          expect(scope.password).toEqual("");
        });

        it('failed request', function(){
          scope.newEmail = "email";
          scope.confirmEmail = "email";
          scope.password = "pw";
          scope.emailForm = {
              newEmail: {},
              confirmEmail: {},
              password: {},
              $valid: true,
              $setPristine: function() {}
          };
          spyOn(scope.emailForm, '$setPristine').and.callFake(function() {});

          httpBackend.expectPUT(scope.apiurl+'/me').respond(500, {'message': "error"});
          scope.updateEmail();
          httpBackend.flush();

          expect(scope.emailForm.$setPristine).toHaveBeenCalled();
          expect(scope.update_email_alert.$options.content).toEqual('error');
        });
      });

});
