#!/bin/bash
#set -e
trap cleanup EXIT

DIR="$HOME/cmb-board/mountBlanc"
WS="$DIR/files/workspace"
. $DIR/backend.cfg

function cleanup
{
	chmod o-rx $WS
	rm -rf $WS
	exit
}

function main
{
	# Give all users write/execute access to workspace
	chmod o+rx $WS
	cd $WS

	# Don't make checker
	mv checker.cpp $DIR
	make -Bs
	mv $DIR/checker.cpp .

	# Run program on small input
	sudo -u worker timeout $(($timeoutLength/3)) ./program < small_input.txt > output.txt
	test=$?
	if [[ $test == 124 ]] ; then
		errorMsg="Program was timed out during correctness test after $(($timeoutLength/3)) seconds!"
		obj="{\"isCorrect\":false, \"msg\":\"$errorMsg\", \"state\": \"CORRECTNESS_TIMEOUT\"}"
		echo 0 1>&3
		echo $obj 1>&3
		exit
	elif [[ $test != 0 ]] ; then
		NUM=$((test-128))
		if (( $NUM < 33 )) ; then
			errorSig=`grep "#define" /usr/include/asm-generic/signal.h | grep "[[:space:]]$NUM$" | expand | tr -s " " |  cut -d" " -f 2`
			signalTxt="Signal: $errorSig\n"
		fi
		output_binary=$(file --mime --brief small_answer.txt | grep charset=binary | wc -l)
		output=`cat output.txt`
		if (( $output_binary == 1 ))
		then
			output="binary file"
		fi
		obj="{\"isCorrect\":false, \"msg\":\"Runtime error in small input! $signalTxt Output:'$output'\", \"state\": \"CORRECTNESS_RUNTIME_ERROR\"}"
		echo 0 1>&3
		echo $obj 1>&3
		exit
	fi
	
	g++ -O2 checker.cpp -o checker
	set +e

	# Run checker on small input
	./checker small_input.txt output.txt small_answer.txt
	test=$?
	set -e
	if [[ $test != 0 ]]; then
		answer_binary=$(file --mime --brief small_answer.txt |grep charset=binary|wc -l)
		output_binary=$(file --mime --brief  cat output.txt |grep charset=binary|wc -l)
		if (( $output_binary == 1 ))
		then
			echo "binary file" > output.txt
			output="binary file"
		fi
		expected=`cat small_answer.txt`
		output=`cat output.txt`
		if [[ $answer_binary == 1 ]]; then
			expected="binary file"
		fi
		errorMsg="Failed small correctness test! Result: $expected was: $output"
		obj="{\"isCorrect\":false, \"msg\":\"$errorMsg\", \"state\": \"CORRECTNESS_BAD_OUTPUT\"}"
		echo 0 1>&3
		echo -e $obj 1>&3
		exit
	fi

	$DIR/dropCache
	$DIR/../EnergyMonitor_v3/EnergyMonitor $samplingInterval 2>&1 | while read line; do d=`date +%s%3N`; echo "$d,$line"; done > $WS/readings.txt &
	monitorPID=$!

	sleep 1
	$DIR/tempAdjustment.sh

    # Run program on main input
	output=`tempfile`
	startTime=`date +%s%3N`
	set +e
	sudo -u worker timeout $timeoutLength ./program < input.txt > $output
	programExitStatus=$?
	set -e
	endTime=`date +%s%3N`

	kill $monitorPID

	if [[ $programExitStatus == 124 ]] ; then
		errorMsg="Program was timed out after $timeoutLength seconds!"
		obj="{\"isCorrect\":false, \"msg\":\"$errorMsg\", \"state\": \"PROGRAM_TIMEOUT\"}"
		echo 0 1>&3
		echo $obj 1>&3
		exit
	elif [[ $programExitStatus != 0 ]] ; then
		NUM=$((test-128))
		errorSig=`grep "#define" /usr/include/asm-generic/signal.h | grep "[[:space:]]$NUM$" | expand | tr -s " " |  cut -d" " -f 2`
		errorMsg="Program crashed! Signal: $errorSig"
		obj="{\"isCorrect\":false, \"msg\":\"$errorMsg\", \"state\": \"PROGRAM_RUNTIME_ERROR\"}"
		echo 0 1>&3
		echo $obj 1>&3
		exit
	fi

	# Print success
	echo 1 1>&3

	time=`echo "scale=2;($endTime - $startTime)/1000" | bc`

	# Handle case where time less than 1. Gives invalid json. Prepend 0
	test=`echo "$time < 1" | bc`
	if [[ $test == 1 ]]; then
		time=0$time
	fi
	cd $DIR

	# Calculate power use
	deltaX=`echo "scale=2; $samplingInterval/1000000" | bc`
	json=`python calculateEnergy.py $WS/readings.txt $deltaX $startTime $endTime $DEBUG`
	power=0

	# Create json string
	obj="$json, \"time\":$time, \"power\":$power}"
	echo $output 1>&3
	echo $obj 1>&3
	exit
}

# Stdout and stderr are redirected to /dev/null if no argument is passed (= NO DEBUG)
exec 3>&1
exec 4>&2
if [ $# -gt 0 ]
then
	DEBUG="True"
else
	DEBUG="False"
	exec 1>/dev/null
	exec 2>/dev/null
fi

main
