import os
import os.path as op
import subprocess
import csv
from os import makedirs

from flask import (Blueprint, current_app, redirect, render_template, request,
                   url_for)
from flask.ext.admin import Admin, AdminIndexView, expose, helpers, BaseView
from flask.ext.admin.contrib.fileadmin import FileAdmin
from flask.ext.admin.contrib.sqla import ModelView
from flask.ext.login import LoginManager, current_user, login_user, logout_user
from werkzeug import secure_filename
from wtforms import fields, form, validators

from database.models import (AdminUser, Group, GroupProblem, Member, Problem,
                             Run, Submission, User, Bulletin, db, Profiling)
from cmb_utils.helpers import delete_submission_files

login_manager = LoginManager()

admin_routes = Blueprint('admin_routes', __name__, None)


@login_manager.user_loader
def load_user(userid):
    return AdminUser.query.filter_by(id=int(userid)).first()


class LoginForm(form.Form):
    username = fields.TextField(validators=[validators.required()])
    password = fields.PasswordField(validators=[validators.required()])

    def validate_username(self, field):
        user = self.get_user()

        if user is None:
            raise validators.ValidationError('Invalid user')

        if not user.check_password(self.password.data):
            raise validators.ValidationError('Invalid password')

    def get_user(self):
        return AdminUser.query.filter_by(name=self.username.data).first()


class MyAdminIndexView(AdminIndexView):
    @expose('/')
    def index(self):
        if not current_user.is_authenticated():
            return redirect(url_for('.login_view'))
        return super(MyAdminIndexView, self).index()

    @expose("/login", methods=["GET", "POST"])
    def login_view(self):
        form = LoginForm(request.form)
        if helpers.validate_form_on_submit(form):
            user = form.get_user()
            login_user(user)

        if current_user.is_authenticated():
            return redirect(url_for(".index"))

        self._template_args['form'] = form
        return super(MyAdminIndexView, self).index()

    @expose("/logout")
    def logout_view(self):
        logout_user()
        return redirect(url_for(".index"))


@admin_routes.route("/")
def index():
    return render_template('index.html')


class MyUserView(ModelView):
    def is_accessible(self):
        return current_user.is_authenticated()

    column_exclude_list = ('password')
    form_excluded_columns = ('groups', 'submissions')

    def __init__(self, session, **kwargs):
        super(MyUserView, self).__init__(User, session, **kwargs)

    def on_model_change(self, form, model, is_created):
        model.set_password(model.password)


class MyAdminUserView(ModelView):
    def is_accessible(self):
        return current_user.is_authenticated()

    column_list = ('name', 'email')

    def __init__(self, session, **kwargs):
        super(MyAdminUserView, self).__init__(AdminUser, session, **kwargs)

    def on_model_change(self, form, model, is_created):
        model.set_password(model.password)


class MyProblemView(ModelView):
    def is_accessible(self):
        return current_user.is_authenticated()

    form_excluded_columns = ('submissions')

    def __init__(self, session, **kwargs):
        super(MyProblemView, self).__init__(Problem, session, **kwargs)

    def on_model_change(self, form, model, is_created):
        path = ("" + current_app.config['UPLOAD_FOLDER'] + "/"
                "" + secure_filename(model.name).lower() + "/problemIO")
        if not op.isdir(path):
            makedirs(path)


class MySubmissionView(ModelView):
    def is_accessible(self):
        return current_user.is_authenticated()

    column_exclude_list = ('profiling_data')

    can_create = False
    column_sortable_list = (
        'name', 'time', 'energy', 'edp', 'goodness_value', 'state',
        'msg', 'submitted', ('problem', Submission.problem_id),
        ('user', Submission.user_id))
    form_overrides = dict(state=fields.SelectField)
    form_args = dict(
        state=dict(
            choices=[('rerun', 'Rerun'), ('compiled', 'Compiled'), ('finished', 'Finished'),
                     ('uploaded', 'Uploaded'), ('in queue', 'In queue'), ('running', 'Running')]
        ))

    def __init__(self, session, **kwargs):
        super(MySubmissionView, self).__init__(Submission, session, **kwargs)

    def on_model_delete(self, model):
        user = User.query.filter_by(id=model.user_id).first()
        problem = Problem.query.filter_by(id=model.problem_id).first()
        delete_submission_files(user, problem, model)


class MyGroupView(ModelView):
    def is_accessible(self):
        return current_user.is_authenticated()

    def __init__(self, session, **kwargs):
        super(MyGroupView, self).__init__(Group, session, **kwargs)


class MyGroupProblemView(ModelView):
    def is_accessible(self):
        return current_user.is_authenticated()

    def __init__(self, session, **kwargs):
        super(MyGroupProblemView, self).__init__(GroupProblem, session, **kwargs)


class MyMemberView(ModelView):
    def is_accessible(self):
        return current_user.is_authenticated()

    def __init__(self, session, **kwargs):
        super(MyMemberView, self).__init__(Member, session, **kwargs)


class MyRunView(ModelView):
    def is_accessible(self):
        return current_user.is_authenticated()

    def __init__(self, session, **kwargs):
        super(MyRunView, self).__init__(Run, session, **kwargs)


class MyProfilingView(ModelView):
    def is_accessible(self):
        return current_user.is_authenticated()

    def __init__(self, session, **kwargs):
        super(MyProfilingView, self).__init__(Profiling, session, **kwargs)


class MyBulletinView(ModelView):
    def is_accessible(self):
        return current_user.is_authenticated()

    form_excluded_columns = ('created_at')

    def __init__(self, session, **kwargs):
        super(MyBulletinView, self).__init__(Bulletin, session, **kwargs)


class MyFileAdmin(FileAdmin):
    def is_accessible(self):
        return current_user.is_authenticated()

    def on_file_upload(self, directory, path, filename):
        nameOfFile = str(op.basename(op.normpath(filename)))

        if nameOfFile.endswith(".zip"):
            command = "unzip " + filename + " -d " + directory
            subprocess.call(command, shell=True)

            # Set file format to unix for all textfiles uploaded
            command = "vim +\"argdo set ff=unix | update\" +wqa " + filename[0:-4] + "/*.txt"
            subprocess.call(command, shell=True)

            # We now assume that this zip always contains checker.cpp, and that it compiles
            command = "g++ -O2 " + filename[0:-4] + "/checker.cpp -o " + filename[0:-4] + "/checker"
            subprocess.call(command, shell=True)

        # Old if-test
        elif nameOfFile.lower() == "checker.cpp":
            command = "g++ -O2 " + filename + " -o " + directory + "/checker"
            subprocess.call(command, shell=True)

        elif not op.isdir(nameOfFile):
            # Set file format to unix for all source and textfiles uploaded
            command = "vim +\"argdo set ff=unix | update\" +wqa " + directory + "/*.txt"
            subprocess.call(command, shell=True)


class MyDBDumpView(BaseView):
    def __init__(self, session, **kwargs):
        self.session = session
        super(MyDBDumpView, self).__init__(**kwargs)

    @expose('/')
    def index(self):
        self.create_db_dump()
        return self.render('admin/dbdump_index.html')

    def create_db_dump(self):
        dbdump_html_path = current_app.config['FLASK_DIR'] + '/source/templates/admin/dbdump_index.html'
        with open(dbdump_html_path, 'wb') as f:
            f.write("{% extends 'admin/master.html' %}\n{% block body %}\n<pre>")

            f.write("\n<h5>Submission</h5>\n")
            self.write_csv_from_model(Submission, f)
            f.write("\n<h5>Run</h5>\n")
            self.write_csv_from_model(Run, f)
            f.write("\n<h5>Profiling</h5>\n")
            self.write_csv_from_model(Profiling, f)
            f.write("\n<h5>Problem</h5>\n")
            self.write_csv_from_model(Problem, f, skip=['description'])
            f.write("\n<h5>User</h5>\n")
            self.write_csv_from_model(User, f, skip=['password'])
            f.write("\n<h5>AdminUser</h5>\n")
            self.write_csv_from_model(AdminUser, f, skip=['password'])
            f.write("\n<h5>Member</h5>\n")
            self.write_csv_from_model(Member, f)
            f.write("\n<h5>GroupProblem</h5>\n")
            self.write_csv_from_model(GroupProblem, f)
            f.write("\n<h5>Group</h5>\n")
            self.write_csv_from_model(Group, f)
            f.write("\n<h5>Bulletin</h5>\n")
            self.write_csv_from_model(Bulletin, f)

            f.write("</pre>\n{% endblock %}")

    def get_columns(self, model, skip):
        return [column for column in model.__mapper__.columns if column.name not in skip]

    def write_csv_from_model(self, model, outfile, skip=None):
        if skip is None: skip = []
        out_csv = csv.writer(outfile)
        records = self.session.query(model).all()
        out_csv.writerow([column.name for column in self.get_columns(model, skip)])
        for curr in records:
            try:
                out_csv.writerow([str(getattr(curr, column.name)).encode('latin-1') for column in self.get_columns(model, skip)])
            except Exception as e:
                print e


admin = Admin(index_view=MyAdminIndexView(), base_template='my_master.html')


def init_admin_views(app):
    admin.add_view(MyUserView(db.session))
    admin.add_view(MyProblemView(db.session))
    admin.add_view(MySubmissionView(db.session))
    admin.add_view(MyGroupView(db.session))
    admin.add_view(MyGroupProblemView(db.session))
    admin.add_view(MyMemberView(db.session))
    admin.add_view(MyRunView(db.session))
    admin.add_view(MyProfilingView(db.session))
    admin.add_view(MyBulletinView(db.session))
    filePath = app.config['UPLOAD_FOLDER']
    admin.add_view(MyFileAdmin(filePath, endpoint="uploads", name="Uploads"))
    admin.add_view(MyAdminUserView(db.session))
    admin.add_view(MyDBDumpView(db.session, name='DB Dump', endpoint='dbdump'))
