import os
import shutil
import zipfile
import subprocess32
from helpers import execute_command


def delete_directory(directory):
    if os.path.exists(directory):
        shutil.rmtree(directory)


def join_paths(paths):
    path = paths[0]
    for p in paths[1:]:
        path = os.path.join(path, p)
    return path


class CompilationError(Exception):
    pass


class CompilationCheck:
    @staticmethod
    def run(problem_name, user_name, submission_name, sub_id, flask_dir, mali_dir):
        submission_basename = submission_name.split('.')[0]
        extension = submission_name.split('.')[-1]
        if extension != 'zip':
            raise CompilationError("File not .zip error.")

        # Create workspace
        workspace = join_paths([mali_dir, 'workspace', str(sub_id), submission_basename])
        try:
            os.makedirs(workspace)
        except OSError:  # Folder already exists
            pass

        try:
            CompilationCheck.unzip_files(join_paths([flask_dir, 'problems', str(problem_name), 'uploads',
                                         str(user_name), submission_name]), workspace)

            # Copy makefile
            shutil.copyfile(join_paths([flask_dir, 'Makefile']),
                            join_paths([workspace, 'Makefile']))

            # Compile (run make)
            execute_command(['make', '-C', workspace, '-Bs'])
        except subprocess32.CalledProcessError as e:
            raise CompilationError('Compilation failed with exit code: ' + str(e.returncode))
        finally:
            CompilationCheck.cleanup(workspace)

    @staticmethod
    def unzip_files(zip_file, workspace):
        try:
            zip_ref = zipfile.ZipFile(zip_file, 'r')
            zip_ref.extractall(workspace)
            zip_ref.close()
        except Exception as e:  # Error unzipping file
            delete_directory(workspace)
            raise CompilationError('Error unzipping files (corrupt .zip file)')

        # If zipfile contains a folder, move out of folder
        for filename in os.listdir(workspace):
            filename_path = join_paths([workspace, filename])
            if os.path.isdir(filename_path):
                for sub_filename in os.listdir(filename_path):
                    sub_filename_path = join_paths([filename_path, sub_filename])
                    os.rename(sub_filename_path, join_paths([workspace, sub_filename]))
                delete_directory(filename_path)

    @staticmethod
    def cleanup(workspace):
        # Remove *.o, Makefile, and executable
        object_files = [o for o in os.listdir(workspace) if o.endswith('.o')]
        for o in object_files:
            os.remove(join_paths([workspace, o]))
        if os.path.exists(join_paths([workspace, 'Makefile'])):
            os.remove(join_paths([workspace, 'Makefile']))
        if os.path.exists(join_paths([workspace, 'program'])):
            os.remove(join_paths([workspace, 'program']))

        delete_directory(os.path.dirname(workspace))


if '__main__' == __name__:
    print CompilationCheck.run('shortest_path', '1_fredripi', 'dijkstra_omp.zip', '1',
                         '/home/fred/dev/cmb/server/cmb-flask',
                         '/home/fred/dev/cmb/Mali_OpenCL_SDK_v1.1.0')
