import csv
import json
import os
import shutil
import subprocess32

from cmb_utils.compilation_check import join_paths, delete_directory, CompilationCheck
from cmb_utils.checker import Checker
from cmb_utils.helpers import execute_command

# Read $APPLICATION_SETTINGS file into dict, since this is called from push wich is a different process and
# does not have app.config
app_settings_path = os.environ['APPLICATION_SETTINGS']
for (key, val) in csv.reader(open(app_settings_path), delimiter='='):
    if key == 'FLASK_DIR':
        flask_dir = val
    elif key == 'UPLOAD_FOLDER':
        upload_dir = val
    if key == 'BOARD_IP':
        board_ip = val


def copy_files_to_folder(files, folder):
    for f in files:
        shutil.copy(f, folder)


class ExecuteOnBackend:
    @staticmethod
    def run(problem_name, user_name, submission_name):
        # Check if board is online
        ExecuteOnBackend.ping_board(board_ip)

        zip_file = join_paths(
            [flask_dir, 'problems', problem_name, 'uploads', str(user_name), submission_name])
        local_dir = submission_name[:-4]
        problem_io_dir = join_paths([upload_dir, problem_name, 'problemIO'])
        user_upload_dir = join_paths([flask_dir, 'problems', problem_name, 'uploads', str(user_name)])

        files = [os.path.join(problem_io_dir, x) for x in
                 ['input.txt', 'small_input.txt', 'small_answer.txt', 'checker.cpp']]
        files.append(os.path.join(flask_dir, 'Makefile'))

        ExecuteOnBackend.copy_files_to_board(board_ip, files, local_dir, zip_file)

        # Run the board's runscript
        output = ExecuteOnBackend.run_script_on_board(board_ip, '/home/climber/cmb-board/mountBlanc/runscript_v2.sh',
                                                  int(os.environ['SERVER_TIMEOUT']))

        lines = output.splitlines()

        try:
            dict_results = json.loads(lines[-1])
        except Exception as a:
            print(a)
            dict_results = {"binary_output": True}

        dict_results['filename'] = str(problem_name)

        if lines[0] == '0':  # Failed small test on board
            return dict_results

        ExecuteOnBackend.copy_file_to_server(board_ip, lines[1], problem_io_dir + '/solution.txt', delete_remote=True)

        correct, goodness = Checker.run(problem_io_dir)
        if os.path.exists(os.path.join(problem_io_dir, 'solution.txt')):
            os.remove(os.path.join(problem_io_dir, 'solution.txt'))

        dict_results['isCorrect'] = correct
        if goodness is not None:
            dict_results['goodness'] = goodness

        return dict_results

    @staticmethod
    def profiling(problem_name, user_name, submission_name):
        # Check if board is online
        ExecuteOnBackend.ping_board(board_ip)

        zip_file = join_paths(
            [flask_dir, 'problems', problem_name, 'uploads', str(user_name), submission_name])
        local_dir = submission_name[:-4]
        problem_io_dir = join_paths([upload_dir, problem_name, 'problemIO'])
        user_upload_dir = join_paths([flask_dir, 'problems', problem_name, 'uploads', str(user_name)])

        files = [os.path.join(problem_io_dir, 'input.txt'), os.path.join(flask_dir, 'Makefile')]

        ExecuteOnBackend.copy_files_to_board(board_ip, files, local_dir, zip_file)

        dict_results = {'success': False}
        try:
            # Run the board's runscript
            data = ExecuteOnBackend.run_script_on_board(board_ip, '/home/climber/cmb-board/mountBlanc/profiling_script.sh',
                                                        int(os.environ['PROFILING_TIMEOUT']))
            dict_results['profiling_data'] = data[:-1]

            ExecuteOnBackend.copy_file_to_server(board_ip, '/home/climber/cmb-board/mountBlanc/flame_graph.svg',
                                                 user_upload_dir + '/' + str(submission_name[:-4]) + '_flame_graph.svg')
            dict_results['success'] = True
        finally:
            return dict_results

    @staticmethod
    def run_script_on_board(board_ip, script_path, timeout):
        return execute_command("ssh climber@" + board_ip +
                               " -x " + script_path, timeout=timeout, shell=True)

    @staticmethod
    def copy_file_to_server(board_ip, file_path, copy_path, delete_remote=False):
        execute_command(
            ['scp', 'climber@' + board_ip + ':' + file_path, copy_path])
        if delete_remote:  # Delete file on board
            execute_command("ssh climber@" + board_ip + " -x rm " + file_path, shell=True)

    @staticmethod
    def ping_board(board_ip):
        try:
            execute_command(['ping', '-q', '-w2', board_ip])
        except subprocess32.CalledProcessError:
            print("Ping error at:", board_ip)
            raise Exception('Ping board error!')

    @staticmethod
    def copy_files_to_board(board_ip, files, local_dir, zip_file):
        # Create tmp folder
        delete_directory(local_dir)
        CompilationCheck.unzip_files(zip_file, local_dir)

        # Copy local files to local tmp folder
        copy_files_to_folder(files, local_dir)

        # Copy to board
        try:
            execute_command(
                ['scp', '-r', local_dir + '/.',
                 'climber@' + board_ip + ':/home/climber/cmb-board/mountBlanc/files/workspace'])
        except subprocess32.CalledProcessError:
            delete_directory(local_dir)
            raise Exception('Error copying files to board.')
        delete_directory(local_dir)


if '__main__' == __name__:
    os.environ['PROFILING_TIMEOUT'] = '2'
    print(ExecuteOnBackend.profiling('shortest_path', '1_fredripi', 'dijkstra_v7.zip'))
