import json

from subprocess32 import PIPE, STDOUT, Popen, TimeoutExpired, CalledProcessError
from datetime import datetime, timedelta
from flask import current_app
from werkzeug import secure_filename
from psutil import Process
import jwt
import os

from database.models import Submission, Problem, User

ALLOWED_EXTENSIONS = {'zip'}


def construct_submission_folder_path(user, problem, submission):
    uploadFolder = current_app.config['UPLOAD_FOLDER'] + '/' + secure_filename(problem.name).lower()
    uploadFolder = uploadFolder + '/uploads/' + str(user.id) + '_' + secure_filename(user.name).lower() + '/'
    uploadFolder = uploadFolder + secure_filename(submission.name)
    return uploadFolder


def delete_submission_files(user, problem, submission):
    if user and problem and submission:
        uploadFolder = construct_submission_folder_path(user, problem, submission)
        if os.path.exists(uploadFolder):
            os.remove(uploadFolder)


def create_jwt_token(app, user):
    payload = {
        'iss': 'localhost',
        'sub': user.id,
        'iat': datetime.now(),
        'exp': datetime.now() + timedelta(hours=1)
    }
    token = jwt.encode(payload, app.config['TOKEN_SECRET'])
    return token.decode('unicode_escape')


def parse_token(app, token):
    return jwt.decode(token, app.config['TOKEN_SECRET'])


def proc_recursive_kill(proc):
    for child in Process(proc.pid).children(recursive=True):
        child.kill()
    proc.kill()


def execute_command(cmd, timeout=None, shell=False):
    proc = Popen(cmd, stdout=PIPE, stderr=STDOUT, shell=shell)

    try:
        out, err = proc.communicate(timeout=timeout)
    except TimeoutExpired as e:
        proc_recursive_kill(proc)
        raise e
    except CalledProcessError as e:
        proc_recursive_kill(proc)
        raise e

    if proc.returncode != 0:
        raise CalledProcessError(proc.returncode, cmd)

    return out


def user_to_json(user):
    return dict(id=user.id, name=user.name, submissions=map(submission_to_json, user.submissions))


def me_to_json(user):
    return dict(id=user.id, name=user.name, email=user.email, submissions=map(submission_to_json, user.submissions))


def submission_to_json(sub):
    if has_flame_graph(sub.id):
        hfg = True
    else:
        hfg = False
    return dict(has_profiling_data=hfg, id=sub.id, problem_id=sub.problem_id, user_id=sub.user_id, user_name=sub.user.name,
                name=sub.name, time=sub.time, energy=sub.energy, state=sub.state, detailed_state=sub.detailed_state,
                msg=sub.msg, goodness=sub.goodness_value, edp=sub.edp, visible=sub.visible,
                profiling_data=sub.profiling_data)


def problem_to_json(prob):
    return dict(id=prob.id, name=prob.name, description=prob.description, allow_profiling=prob.allow_profiling,
                submissions=map(submission_to_json, prob.submissions), goodness=prob.goodness_name)


def group_to_json(group):
    return dict(id=group.id, name=group.name, description=group.description, members=map(member_to_json, group.members),
                problems=filter((lambda x: x is not None), map(group_problem_to_json, group.problems)),
                public=group.public)


def member_to_json(member):
    return dict(id=member.id, user_id=member.user_id, username=member.user.name, group_id=member.group_id,
                role=member.role)


def group_problem_to_json(group_problem):
    if group_problem.problem.visible:
        group_problem_json = dict(id=group_problem.id,
                                  problem_id=group_problem.problem_id,
                                  problem_name=group_problem.problem.name,
                                  group_id=group_problem.group_id)
        if hasattr(group_problem, 'submissions'):
            sub_json = filter((lambda x: x is not None), map(submission_to_json, group_problem.submissions))
            group_problem_json['submissions'] = sub_json
        return group_problem_json
    else:
        return None


def my_group_to_json(group, user_id):
    for member in group.members:
        if member.user_id == user_id:
            role = member.role
    return dict(id=group.id,
                name=group.name,
                role=role,
                problems=filter((lambda x: x is not None), map(group_problem_to_json, group.problems)))


def group_with_submission_to_json(group, submissions):
    for problem in group.problems:
        problem.submissions = filter_submissions_by_problem(submissions, problem.problem_id)
    return group_to_json(group)


def bulletin_to_json(bulletin):
    return dict(id=bulletin.id, content=bulletin.content, valid_until=bulletin.valid_until,
                removable=bulletin.removable)


def filter_submissions_by_problem(submissions, problem_id):
    return filter((lambda s: s.problem_id == problem_id), submissions)


def allowed_file(filename):
    return '.' in filename and \
           filename.rsplit('.', 1)[1] in ALLOWED_EXTENSIONS


def has_flame_graph(sub_id):
    sub = Submission.query.filter_by(id=sub_id).first()
    problem = Problem.query.filter_by(id=sub.problem_id).first()
    user = User.query.filter_by(id=sub.user_id).first()

    sub_basename = sub.name.split('.')[0]
    upload_dir = current_app.config["FLASK_DIR"] + '/problems/' + secure_filename(problem.name).lower() +\
        '/uploads/' + str(sub.user_id) + "_" + secure_filename(user.name).lower()
    if os.path.exists(os.path.join(upload_dir, sub_basename + '_flame_graph.svg')):
        return upload_dir, sub_basename
    else:
        return False
