from datetime import datetime

from flask.ext.sqlalchemy import SQLAlchemy
from werkzeug.security import check_password_hash, generate_password_hash


db = SQLAlchemy()


class User(db.Model):
    __tablename__ = 'users'
    id = db.Column(db.Integer, primary_key=True)
    name = db.Column(db.String(50), unique=False)
    email = db.Column(db.String(120))
    password = db.Column(db.String(120))
    submissions = db.relationship("Submission", backref="user")
    created = db.Column(db.DateTime)

    def __init__(self, name=None, password=None, email="No Email", submissions=[]):
        self.name = name
        self.email = email
        if password:
            self.set_password(password)
        self.submissions = submissions
        self.created = datetime.now()

    def set_password(self, password):
        self.password = generate_password_hash(password)

    def check_password(self, password):
        return check_password_hash(self.password, password)

    def __unicode__(self):
        return self.name


class AdminUser(db.Model):
    __tablename__ = 'admins'
    id = db.Column(db.Integer, primary_key=True)
    name = db.Column(db.String(50), unique=False)
    email = db.Column(db.String(120))
    password = db.Column(db.String(120))

    def __init__(self, name=None, password=None, email="No Email"):
        self.name = name
        self.email = email
        if password:
            self.set_password(password)

    def set_password(self, password):
        self.password = generate_password_hash(password)

    def check_password(self, password):
        return check_password_hash(self.password, password)

    def is_active(self):
        return True

    def is_authenticated(self):
        return True

    def is_anonymous(self):
        return False

    def get_id(self):
        try:
            return unicode(self.id)
        except AttributeError:
            raise NotImplementedError('No `id` attribute - override `get_id`')

    def __unicode__(self):
        return self.name


class Submission(db.Model):
    __tablename__ = 'submissions'
    id = db.Column(db.Integer, primary_key=True)
    name = db.Column(db.String(120), unique=False)
    time = db.Column(db.Float)
    energy = db.Column(db.Float)
    edp = db.Column(db.Float)
    goodness_value = db.Column(db.Float)
    state = db.Column(db.String(30), unique=False)
    detailed_state = db.Column(db.String(30), unique=False)
    user_id = db.Column(db.Integer, db.ForeignKey('users.id'))
    problem_id = db.Column(db.Integer, db.ForeignKey('problems.id'))
    msg = db.Column(db.Text, unique=False)
    submitted = db.Column(db.DateTime)
    runs = db.relationship('Run', cascade="all, delete-orphan", backref="submission")
    profilings = db.relationship('Profiling', cascade="all, delete-orphan", backref="submission")
    visible = db.Column(db.Boolean)
    profiling_data = db.Column(db.String(1024), unique=False)

    def __init__(self, name=None, problem_id=None, user_id=None, state=None, detailed_state=None, time=None,
                 energy=None, edp=None, visible=True):
        self.name = name
        self.user_id = user_id
        self.problem_id = problem_id
        self.state = state
        self.time = time
        self.energy = energy
        self.edp = edp
        self.submitted = datetime.now()
        self.visible = visible

    def __unicode__(self):
        return self.name


class Run(db.Model):
    __tablename__ = 'runs'
    id = db.Column(db.Integer, primary_key=True)
    submission_id = db.Column(db.Integer, db.ForeignKey('submissions.id'))
    enqueued = db.Column(db.DateTime)
    dequeued = db.Column(db.DateTime)
    finished = db.Column(db.DateTime)
    runtime = db.Column(db.Float)
    energy = db.Column(db.Float)
    goodness_value = db.Column(db.Float)


class Profiling(db.Model):
    __tablename__ = 'profilings'
    id = db.Column(db.Integer, primary_key=True)
    submission_id = db.Column(db.Integer, db.ForeignKey('submissions.id'))
    enqueued = db.Column(db.DateTime)
    dequeued = db.Column(db.DateTime)
    finished = db.Column(db.DateTime)
    profiling_data = db.Column(db.String(1024), unique=False)


class Problem(db.Model):
    __tablename__ = 'problems'
    id = db.Column(db.Integer, primary_key=True)
    name = db.Column(db.String(120), unique=False)
    description = db.Column(db.Text)
    submissions = db.relationship('Submission', backref="problem")
    created = db.Column(db.DateTime)
    goodness_name = db.Column(db.String(120))
    visible = db.Column(db.Boolean)
    allow_profiling = db.Column(db.Boolean)

    def __init__(self, name=None, description=None, goodness_name=None, visible=True, allow_profiling=True):
        self.name = name
        self.description = description
        self.created = datetime.now()
        self.goodness_name = goodness_name
        self.visible = visible
        self.allow_profiling = allow_profiling

    def __unicode__(self):
        return self.name


class Member(db.Model):
    __tablename__ = 'members'
    id = db.Column(db.Integer, primary_key=True)
    user_id = db.Column(db.Integer, db.ForeignKey('users.id'))
    group_id = db.Column(db.Integer, db.ForeignKey('groups.id'))
    role = db.Column(db.String(20))
    created = db.Column(db.DateTime)
    group = db.relationship('Group', backref='members')
    user = db.relationship('User', backref='groups')

    def __init__(self, user=None, group=None, role="member"):
        self.user = user
        self.group = group
        self.created = datetime.now()
        self.role = role

    def __unicode__(self):
        return self.group.name + "; " + self.user.name + "; " + self.role


class GroupProblem(db.Model):
    __tablename__ = 'group_problems'
    id = db.Column(db.Integer, primary_key=True)
    problem_id = db.Column(db.Integer, db.ForeignKey('problems.id'))
    group_id = db.Column(db.Integer, db.ForeignKey('groups.id'))
    visible = db.Column(db.DateTime)
    deadline = db.Column(db.DateTime)
    created = db.Column(db.DateTime)
    group = db.relationship('Group', backref='problems')
    problem = db.relationship('Problem')

    def __init__(self, problem=None, group=None, deadline=None, visible=None):
        self.problem = problem
        self.group = group
        self.deadline = deadline
        self.visible = visible if visible else datetime.now()
        self.created = datetime.now()

    def __unicode__(self):
        return self.group.name + "; " + self.problem.name


class Group(db.Model):
    __tablename__ = 'groups'
    id = db.Column(db.Integer, primary_key=True)
    name = db.Column(db.String(120), unique=False)
    description = db.Column(db.Text)
    created = db.Column(db.DateTime)
    public = db.Column(db.Boolean)

    def __init__(self, name=None, description=None, public=False):
        self.name = name
        self.description = description
        self.created = datetime.now()
        self.public = public

    def __unicode__(self):
        return self.name


class Bulletin(db.Model):
    __tablename__ = 'bulletin'
    id = db.Column(db.Integer, primary_key=True)
    content = db.Column(db.Text)
    valid_until = db.Column(db.DateTime)
    removable = db.Column(db.Boolean)
    created = db.Column(db.DateTime)

    def __init__(self, content=None, valid_until=None, removable=False):
        self.content = content
        self.valid_until = valid_until
        self.removable = removable
        self.created = datetime.now()
