from flask import Blueprint, g, jsonify, request
from sqlalchemy import and_

from cmb_utils.helpers import group_to_json, my_group_to_json, group_with_submission_to_json
from cmb_utils.wrappers import myWrappers
from database.models import Group, GroupProblem, Member, Problem, Submission, User, db

group_routes = Blueprint('group_routes', __name__, None)


def validate_request(group, obj):
    if not obj:
        response = jsonify(message='Object does not exist.')
        response.status_code = 400
        return response
    if not group:
        response = jsonify(message='Group does not exist.')
        response.status_code = 400
        return response
    leader = Member.query.filter(and_(Member.user_id == g.user_id, Member.group == group)).first()
    if not leader or not leader.role == "leader":
        response = jsonify(message='Must have group leader permissions.')
        response.status_code = 401
        return response
    response = jsonify(message='OK')
    response.status_code = 200
    return response


@group_routes.route('/api/leader/<int:group_id>', methods=['GET'])
@myWrappers.login_required
def is_leader(group_id):
    group = Group.query.filter_by(id=group_id).first()
    if not group:
        response = jsonify(message='Group does not exist.')
        response.status_code = 400
        return response
    me = Member.query.filter(and_(Member.user_id == g.user_id, Member.group_id == group_id)).first()
    if not me or not me.role == "leader":
        response = jsonify(message='Not Leader')
        response.status_code = 200
        return response
    response = jsonify(message='Leader')
    response.status_code = 200
    return response


@group_routes.route('/api/groups/public', methods=['GET'])
def get_public_groups():
    groups = Group.query.filter_by(public=True).all()
    return jsonify({'groups': map(group_to_json, groups)})


@group_routes.route('/api/mygroups', methods=['GET'])
@myWrappers.login_required
def get_my_groups():
    me = User.query.filter_by(id=g.user_id).first()
    groups = Group.query.all()
    mygroups = []
    user_ids = []
    for group in groups:
        for member in group.members:
            if member.user == me:
                mygroups.append(group)
                user_ids.append(g.user_id)
    return jsonify({'groups': map(my_group_to_json, mygroups, user_ids)})


@group_routes.route('/api/group/<int:group_id>', methods=['GET'])
@myWrappers.fetch_login_status
def get_group(group_id):
    if g.user_id:
        group = Group.query.filter_by(id=group_id).first()
    else:
        group = Group.query.filter(and_(Group.id == group_id, Group.public)).first()
    if not group:
        response = jsonify(message='Group does not exist.')
        response.status_code = 400
        return response

    with_submissions = request.args.get('with-submissions', 0, type=int)
    if with_submissions:
        users = []
        is_leader = False
        is_member = False
        for member in group.members:
            if g.user_id and member.user_id == g.user_id:
                is_leader = (member.role.lower() == 'leader')
                is_member = (member.role.lower() == 'member')
            users.append(member.user_id)
        if not g.user_id and (not is_member or not is_leader):
            return jsonify({'group': group_to_json(group)})
        problems = []
        for problem in group.problems:
            problems.append(problem.problem_id)
        if is_leader:
            submissions = Submission.query.filter(
                and_(Submission.user_id.in_(users),
                     Submission.problem_id.in_(problems))).all()
        else:
            submissions = Submission.query.filter(
                and_(Submission.user_id.in_(users),
                     Submission.problem_id.in_(problems))).filter_by(visible=True).all()
        return jsonify({'group': group_with_submission_to_json(group, submissions)})
    return jsonify({'group': group_to_json(group)})


@group_routes.route('/api/group/<int:group_id>', methods=['PUT'])
@myWrappers.login_required
def toggle_group_public(group_id):
    group = Group.query.filter_by(id=group_id).first()
    if not group:
        response = jsonify(message='Group does not exist.')
        response.status_code = 400
        return response
    me = Member.query.filter(and_(Member.user_id == g.user_id, Member.group_id == group_id)).first()
    if not me or not me.role == "leader":
        response = jsonify(message='Not Authorized to toggle group public')
        response.status_code = 401
        return response
    group.public = not group.public
    db.session.commit()
    return jsonify({'group': group_to_json(group)})


@group_routes.route('/api/group/create', methods=['POST'])
@myWrappers.login_required
def create_new_group():
    description = request.json['description'] if 'description' in request.json else None
    if 'name' not in request.json:
        response = jsonify(message='Group must have a name.')
        response.status_code = 400
        return response
    name = request.json['name']
    group = Group(name, description)
    user = User.query.filter_by(id=g.user_id).first()
    db.session.add(group)
    db.session.add(user)
    member = Member(user, group, "leader")
    db.session.add(member)
    db.session.commit()
    return jsonify({'group': group_to_json(group)})


@group_routes.route('/api/group/<int:group_id>', methods=['DELETE'])
@myWrappers.login_required
def delete_group(group_id):
    group = Group.query.filter_by(id=group_id).first()
    if not group:
        response = jsonify(message='Group does not exist.')
        response.status_code = 400
        return response
    me = Member.query.filter(and_(Member.user_id == g.user_id, Member.group_id == group_id)).first()
    if not me or not me.role == "leader":
        response = jsonify(message='Not Authorized to delete group ' + str(group_id))
        response.status_code = 401
        return response
    db.session.delete(group)
    db.session.commit()
    return jsonify(message='Group ' + str(group_id) + ' deleted.')


@group_routes.route('/api/group/<int:group_id>/member/<int:user_id>', methods=['POST'])
@myWrappers.login_required
def add_member(group_id, user_id):
    user = User.query.filter_by(id=user_id).first()
    group = Group.query.filter_by(id=group_id).first()
    response = validate_request(group, user)
    if response.status_code != 200:
        return response
    member = Member(user, group)
    db.session.add(member)
    db.session.commit()
    return jsonify({'group': group_to_json(group)})


@group_routes.route('/api/group/leave/<int:group_id>', methods=['DELETE'])
@myWrappers.login_required
def leave_group(group_id):
    me = Member.query.filter(and_(Member.user_id == g.user_id, Member.group_id == group_id)).first()
    if not me:
        response = jsonify(message='Not a member.')
        response.status_code = 400
        return response
    if me.role == 'leader':
        response = jsonify(message='Leader cannot leave group.')
        response.status_code = 400
        return response
    db.session.delete(me)
    db.session.commit()
    group = Group.query.filter_by(id=group_id).first()
    return jsonify({'group': group_to_json(group)})


@group_routes.route('/api/group/join/<int:group_id>', methods=['POST'])
@myWrappers.login_required
def join_group(group_id):
    member = Member.query.filter(and_(Member.user_id == g.user_id, Member.group_id == group_id)).first()
    if member:
        response = jsonify(message='Already a member.')
        response.status_code = 400
        return response
    me = User.query.filter_by(id=g.user_id).first()
    group = Group.query.filter_by(id=group_id).first()
    if not group:
        response = jsonify(message='Group does not exist.')
        response.status_code = 400
        return response
    if not group.public:
        response = jsonify(message='Group is not public.')
        response.status_code = 400
        return response
    member = Member(me, group)
    db.session.add(member)
    db.session.commit()
    return jsonify({'group': group_to_json(group)})


@group_routes.route('/api/group/<int:group_id>/member/<int:user_id>', methods=['DELETE'])
@myWrappers.login_required
def delete_member(group_id, user_id):
    user = User.query.filter_by(id=user_id).first()
    group = Group.query.filter_by(id=group_id).first()
    response = validate_request(group, user)
    if response.status_code != 200:
        return response
    member = Member.query.filter(and_(Member.user_id == user_id, Member.group_id == group_id)).first()
    db.session.delete(member)
    db.session.commit()
    return jsonify({'group': group_to_json(group)})


@group_routes.route('/api/group/<int:group_id>/problem/<int:problem_id>', methods=['POST'])
@myWrappers.login_required
def add_problem(group_id, problem_id):
    problem = Problem.query.filter_by(id=problem_id).first()
    group = Group.query.filter_by(id=group_id).first()
    response = validate_request(group, problem)
    if response.status_code != 200:
        return response
    group_prob = GroupProblem(problem, group)
    db.session.add(group_prob)
    db.session.commit()
    return jsonify({'group': group_to_json(group)})


@group_routes.route('/api/group/<int:group_id>/problem/<int:problem_id>', methods=['DELETE'])
@myWrappers.login_required
def delete_problem(group_id, problem_id):
    problem = Problem.query.filter_by(id=problem_id).first()
    group = Group.query.filter_by(id=group_id).first()
    response = validate_request(group, problem)
    if response.status_code != 200:
        return response
    group_prob = GroupProblem.query.filter(
        and_(GroupProblem.problem_id == problem_id, GroupProblem.group_id == group_id)).first()
    db.session.delete(group_prob)
    db.session.commit()
    return jsonify({'group': group_to_json(group)})
