import json
import os
import subprocess
from datetime import datetime
from Queue import Queue

from flask import Blueprint, current_app, g, jsonify, request, send_from_directory
from werkzeug import secure_filename

from cmb_utils.compilation_check import CompilationCheck, CompilationError, join_paths
from cmb_utils.helpers import (allowed_file, submission_to_json, user_to_json,
                               delete_submission_files, has_flame_graph)
from cmb_utils.wrappers import myWrappers
from database.models import Run, Submission, User, Problem, db, Profiling
from server import socketio

submission_routes = Blueprint('submission_routes', __name__, None)

queue = Queue()


def queue_size():
    return queue.qsize()


@submission_routes.route('/api/submission/<int:sub_id>', methods=['DELETE'])
@myWrappers.login_required
def delete_submission(sub_id):
    sub = Submission.query.filter_by(id=sub_id).first()
    if not sub:
        response = jsonify(message="Submission does not exist")
        response.status_code = 400
        return response
    elif g.user_id != sub.user_id:
        response = jsonify(message="Not authorized to delete submission")
        response.status_code = 401
        return response
    problem = Problem.query.filter_by(id=sub.problem_id).first()
    user = User.query.filter_by(id=g.user_id).first()
    delete_submission_files(user, problem, sub)
    db.session.delete(sub)
    db.session.commit()
    socketio.emit('submission-deleted', {'sub_id': sub_id}, room=str(sub.problem_id), namespace='/cmb')
    return jsonify(message='Submission ' + str(sub_id) + ' deleted.')


@submission_routes.route('/api/submission/<int:sub_id>', methods=['GET'])
def get_submission(sub_id):
    # should improve security on this route
    sub = Submission.query.filter_by(id=sub_id).first()
    return jsonify({'submission': submission_to_json(sub)})


@submission_routes.route('/api/flamegraph/<int:sub_id>', methods=['GET'])
def get_flame_graph(sub_id):
    upload_dir, sub_basename = has_flame_graph(sub_id)
    return send_from_directory(upload_dir, sub_basename + '_flame_graph.svg')


def saveFile(file, prob_name, folderName, filename):
    path = current_app.config['UPLOAD_FOLDER'] + "/" + prob_name + "/uploads/" + folderName
    if not os.path.isdir(path):
        os.makedirs(path)
    file.save(os.path.join(path, filename))


@submission_routes.route('/api/upload/<int:problemId>/<string:prob_name>/<string:file_name>', methods=['POST'])
@myWrappers.login_required
def upload_file(problemId, prob_name, file_name):
    if request.method == 'POST':
        file = request.files['file']
        user = User.query.filter_by(id=g.user_id).first()
        if file and allowed_file(file_name):
            filename = secure_filename(file_name)
            prob_name = secure_filename(prob_name).lower()
            username = secure_filename(user.name).lower()
            saveFile(file, prob_name, str(g.user_id) + "_" + username, filename)
            uploaded_file = Submission(name=filename, problem_id=problemId, user_id=g.user_id, state="uploaded")
            db.session.add(uploaded_file)
            db.session.commit()
            socketio.emit('submission-uploaded', {'sub_id': uploaded_file.id}, room=str(problemId),
                          namespace='/cmb')
            user = User.query.filter_by(id=g.user_id).first()
            return jsonify({'user': user_to_json(user)})
        else:
            response = jsonify(message='Filetype not supported. Must be ".zip".')
            response.status_code = 400
            return response


@submission_routes.route('/api/compile/<string:problemName>/<int:sub_id>', methods=['POST'])
@myWrappers.login_required
def compile(sub_id, problemName):
    user = User.query.filter_by(id=g.user_id).first()
    submission = Submission.query.filter_by(id=sub_id).first()
    problemName = secure_filename(problemName).lower()
    userName = secure_filename(user.name).lower()
    print str(sub_id)
    try:
        CompilationCheck.run(problemName, str(g.user_id) + "_" + userName, submission.name, str(sub_id),
                             current_app.config["FLASK_DIR"], current_app.config["MALI_DIR"])
        submission.state = "compiled"
    except CompilationError as e:
        submission.state = "failed"
        submission.detailed_state = "COMPILATION_FAILED"
        submission.msg = str(e)

    db.session.commit()
    return jsonify({'user': user_to_json(user)})


@submission_routes.route('/api/submission/<int:sub_id>', methods=['PUT'])
@myWrappers.login_required
def toggle_visible(sub_id):
    submission = Submission.query.filter_by(id=sub_id).first()
    if submission is not None:
        submission.visible = not submission.visible
        db.session.commit()
        response = jsonify(message='Submission visibility toggled.')
        response.status_code = 200
        return response
    else:
        response = jsonify(message='Submission does not exist.')
        response.status_code = 400
        return response


@submission_routes.route('/api/profiling', methods=['PUT'])
@myWrappers.sameIP_required
def profiling():
    dict_results = json.loads(request.data)
    sub = Submission.query.filter_by(id=dict_results['submissionId']).first()
    profiling = Profiling.query.filter_by(id=dict_results['runId']).first()
    profiling.finished = datetime.now()
    sub.state = 'finished'
    if dict_results['success']:
        print dict_results['profiling_data']
        profiling.profiling_data = dict_results['profiling_data']
        sub.profiling_data = dict_results['profiling_data']
        socketio.emit('profiling-finished', {'sub_id': sub.id}, room=str(sub.problem_id), namespace='/cmb')
    else:
        socketio.emit('profiling-failed', {'sub_id': sub.id}, room=str(sub.problem_id), namespace='/cmb')
    db.session.commit()
    response = jsonify(message="Ack")
    response.status_code = 200
    return response


@submission_routes.route('/api/result', methods=['PUT'])
@myWrappers.sameIP_required
def result():
    dict_results = json.loads(request.data)
    sub = Submission.query.filter_by(id=dict_results['submissionId']).first()
    run = Run.query.filter_by(id=dict_results['runId']).first()
    run.finished = datetime.now()

    if dict_results['isCorrect'] in ['true', True, 'True']:
        run.runtime = sub.time = dict_results['time']
        run.energy = sub.energy = dict_results['energy']
        sub.edp = sub.time * sub.energy
        sub.power = dict_results['power']
        sub.msg = None
        sub.state = 'finished'
        if 'goodness' in dict_results:
            run.goodness_value = sub.goodness_value = dict_results['goodness']
    else:
        sub.state = 'failed'
        if 'state' in dict_results:
            sub.detailed_state = dict_results['state']
        sub.msg = dict_results['msg']

    db.session.commit()
    socketio.emit('submission-finished', {'sub_id': sub.id}, room=str(sub.problem_id), namespace='/cmb')
    response = jsonify(message="Ack")
    response.status_code = 200
    return response


@submission_routes.route('/api/enqueue/<string:problemName>/<int:subId>/<string:profiling>', methods=['POST'])
@myWrappers.login_required
def enqeue(problemName, subId, profiling):
    print 'enqueuing....'
    user = User.query.filter_by(id=g.user_id).first()
    file = Submission.query.filter_by(id=subId).first()
    problemName = secure_filename(problemName).lower()

    is_profiling = profiling in ['True', 'true', True]
    if not is_profiling:
        file.state = "in queue"
        run = Run(enqueued=datetime.now(), submission_id=subId)
        db.session.add(run)
        db.session.commit()
        runId = run.id
    else:
        # The state is set to profiling although it might be in a queue, so that it is not shown in run queue
        # in the frontend (it is technically in the same queue).
        file.state = "profiling"
        profiling = Profiling(enqueued=datetime.now(), submission_id=subId)
        db.session.add(profiling)
        db.session.commit()
        runId = profiling.id

    queue.put({'filename': file.name, 'problemName': problemName,
               'userName': str(g.user_id) + "_" + secure_filename(user.name).lower(), 'submissionId': subId,
               'runId': runId, 'profiling': is_profiling})
    user = User.query.filter_by(id=g.user_id).first()
    socketio.emit('submission-enqueued', {'sub_id': subId, 'queue_number': queue.qsize()},
                  room=str(file.problem_id), namespace='/cmb')
    return jsonify({'user': user_to_json(user)})



@submission_routes.route('/api/dequeue', methods=['GET'])
@myWrappers.sameIP_required
def dequeue():
    if queue.empty():
        return jsonify({'filename': None})
    print "dequeueing..."

    top = queue.get()

    if not top['profiling']:
        submission = Submission.query.filter_by(id=top['submissionId']).first()
        submission.state = "running"
        run = Run.query.filter_by(id=top['runId']).first()
        run.dequeued = datetime.now()
        db.session.commit()
        print(submission.problem_id)
        socketio.emit('submission-dequeued', {'sub_id': submission.id, 'running_time': 2},
                      room=str(submission.problem_id), namespace='/cmb')
    else:
        profiling = Profiling.query.filter_by(id=top['runId']).first()
        profiling.dequeued = datetime.now()
        db.session.commit()

    return jsonify(top)
