import random
import string
from base64 import b64decode

from flask import Blueprint, current_app, g, jsonify, request

from cmb_utils.helpers import create_jwt_token, me_to_json, user_to_json
from cmb_utils.mail import send_mail
from cmb_utils.wrappers import myWrappers
from database.models import User, db

user_routes = Blueprint('user_routes', __name__, None)


@user_routes.route('/auth/login', methods=['POST'])
def login():
    name = request.json['name'].capitalize()
    user = User.query.filter_by(name=name).first()
    if not user or not user.check_password(b64decode(request.json['password'])):
        response = jsonify(message='Wrong Username or Password')
        response.status_code = 401
        return response
    token = create_jwt_token(current_app, user)
    return jsonify(token=token)


@user_routes.route('/auth/signup', methods=['POST'])
def signup():
    name = request.json['name'].capitalize()
    existing_user = User.query.filter_by(name=name).first()
    if existing_user:
        response = jsonify(message='Username already exist!')
        response.status_code = 400
        return response
    existing_email = User.query.filter_by(email=request.json['email'].lower()).first()
    if existing_email:
        response = jsonify(message='User with email already exist!')
        response.status_code = 400
        return response
    user = User(name=name, email=request.json['email'].lower(), password=b64decode(request.json['password']))
    db.session.add(user)
    db.session.commit()
    msg = ("<p>Welcome, " + name + "!</p><p>You have succesfully signed up to the Climbing Mont Blanc system.</p>"
           "<p>This is an automatically generated email.</p>"
           "<p>Please contact us at this email if you have any questions.</p>"
           "<br><p>Best regards,</p><p>The Climbing Mont Blanc Team</p>")
    subject = "Welcome to Climbing Mont Blanc"
    send_mail(msg, subject, [user.email])
    token = create_jwt_token(current_app, user)
    return jsonify(token=token)


@user_routes.route('/api/me', methods=['GET'])
@myWrappers.login_required
def me():
    user = User.query.filter_by(id=g.user_id).first()
    return jsonify({'user': me_to_json(user)})


@user_routes.route('/api/me', methods=['PUT'])
@myWrappers.login_required
def update_me():
    user = User.query.filter_by(id=g.user_id).first()
    if user.check_password(b64decode(request.json['password'])):
        if request.json['type'] == 'password':
            user.set_password(b64decode(request.json['new_password']))
            db.session.commit()
            new_user = User.query.filter_by(id=g.user_id).first()
            msg = ("<p>Hi, " + new_user.name + "!</p><p></p><p>You have succesfully changed password.</p><br>"
                   "<p>This is an automatically generated email.</p>"
                   "<p>Please contact us at this email if you have any questions.</p><br>"
                   "<p>Best regards,</p><p>The Climbing Mont Blanc Team</p>")
            subject = "Climbing Mont Blanc: Password Changed"
            send_mail(msg, subject, [user.email])
            return jsonify({'user': me_to_json(new_user)})
        elif request.json['type'] == 'email':
            existing_email = User.query.filter_by(email=request.json['email'].lower()).first()
            if existing_email:
                response = jsonify(message='User with email already exist!')
                response.status_code = 400
                return response
            user.email = request.json['email'].lower()
            db.session.commit()
            new_user = User.query.filter_by(id=g.user_id).first()
            msg = ("<p>Hi, " + new_user.name + "!</p><p>You have succesfully changed your registered email.</p>"
                   "<p>This is an automatically generated email.</p>"
                   "<p>Please contact us at this email if you have any questions.</p><br>"
                   "<p>Best regards,</p><p>The Climbing Mont Blanc Team</p>")
            subject = "Climbing Mont Blanc: Email Changed"
            send_mail(msg, subject, [user.email])
            return jsonify({'user': me_to_json(new_user)})
        else:
            response = jsonify(message='Type not supported')
            response.status_code = 400
            return response
    else:
        response = jsonify(message='Wrong Password')
        response.status_code = 401
        return response


@user_routes.route('/api/forgot_password', methods=['POST'])
def forgot_password():
    name = request.json['username'].capitalize()
    user = User.query.filter_by(name=name).first()
    if not user or user.email != request.json['email']:
        response = jsonify(message='Username/Email does not match any record.')
        response.status_code = 400
        return response
    new_pw = ''.join(random.SystemRandom().choice(string.ascii_uppercase + string.digits) for _ in range(8))
    user.set_password(new_pw)
    db.session.commit()
    msg = ("<p>Hi, " + user.name + "!</p><p>New password: " + new_pw + "</p>"
           "<p>Please go to profile to set a new password as soon as possible.</p>"
           "<p>This is an automatically generated email.</p>"
           "<p>Please contact us at this email if you have any questions.</p><br>"
           "<p>Best regards,</p><p>The Climbing Mont Blanc Team</p>")
    subject = "Climbing Mont Blanc: Password reset"
    send_mail(msg, subject, [user.email])
    return jsonify(email=user.email)


@user_routes.route('/api/users', methods=['GET'])
def get_users():
    users = User.query.all()
    return jsonify({'users': map(user_to_json, users)})


@user_routes.route('/api/user/<int:user_id>', methods=['GET'])
def get_user(user_id):
    user = User.query.filter(User.id == user_id).first()
    return jsonify({'user': user_to_json(user)})
