#!flask/bin/python
import os
import threading

from flask import Flask
from flask.ext.migrate import Migrate, MigrateCommand
from flask.ext.script import Manager

from flask.ext.socketio import SocketIO, rooms, leave_room, emit
from werkzeug.contrib.fixers import ProxyFix

from cmb_utils.mail import mail
from cmb_utils.wrappers import myWrappers
from database.models import db
from admin.admin import admin, login_manager, init_admin_views, admin_routes
from push import Push

app = Flask(__name__)

app.config['TOKEN_SECRET'] = os.environ['CMB_TOKEN_SECRET']
app.config.from_envvar('APPLICATION_SETTINGS')
app.config['MAIL_USERNAME'] = os.environ['CMB_MAIL_USERNAME']
app.config['MAIL_PASSWORD'] = os.environ['CMB_MAIL_PASSWORD']
app.config['SQLALCHEMY_DATABASE_URI'] = os.environ['SQLALCHEMY_DATABASE_URI']
app.config['SERVER_TIMEOUT'] = os.environ['SERVER_TIMEOUT']
app.secret_key = os.environ['CMB_SECRET_KEY']
app.config['SECRET_KEY'] = os.environ['CMB_SECRET_KEY']

admin.init_app(app)
login_manager.init_app(app)
db.init_app(app)

app.register_blueprint(admin_routes)

init_admin_views(app)

debugging_enabled = False
engineio_logger_enabled = False

if app.config['VERSION'] == 'dev':
    # debugging_enabled = True
    engineio_logger_enabled = True


socketio = SocketIO(async_mode='gevent', engineio_logger=engineio_logger_enabled)


def create_app(self, push=True):
    from routes.groups import group_routes
    from routes.problems import problem_routes
    from routes.submissions import submission_routes
    from routes.users import user_routes
    from routes.bulletin import bulletin_routes

    mail.init_app(app)
    myWrappers.init_app(app)

    app.register_blueprint(user_routes)
    app.register_blueprint(problem_routes)
    app.register_blueprint(submission_routes)
    app.register_blueprint(group_routes)
    app.register_blueprint(bulletin_routes)

    Migrate(app, db)

    app.wsgi_app = ProxyFix(app.wsgi_app)
    socketio.init_app(app)

    @socketio.on('connect', namespace='/cmb')
    def connect():
        emit('message', {'msg': 'Connected'})

    @socketio.on('disconnect',  namespace='/cmb')
    def disconnect():
        for room in rooms():
            leave_room(room)
        emit('message', {'msg': 'Disconnected'})

    from events.problems import problem_events
    app.register_blueprint(problem_events)

    configure_app(app)

    if push:
        threading.Timer(5, run_push).start()

    return app


def configure_manager(app):
    manager = Manager(app)
    manager.add_command('db', MigrateCommand)
    return manager


def configure_app(app):
    @app.after_request
    def after_request(response):
        response.headers.add('Access-Control-Allow-Origin', '*')
        response.headers.add('Access-Control-Allow-Headers', 'Content-Type,Authorization')
        response.headers.add('Access-Control-Allow-Methods', 'GET,PUT,POST,DELETE,OPTION')
        return response

    @app.teardown_appcontext
    def shutdown_session(exception=None):
        db.session.remove()


def run_push():
    Push.run()
    threading.Timer(2, run_push).start()


def init_db():
    db.init_app(app)
    db.create_all()


def delete_db():
    db.drop_all()
