import subprocess
import os
import json
import zipfile

from flask.ext.testing import TestCase
from werkzeug import secure_filename
from shutil import rmtree

from cmb_utils.helpers import create_jwt_token
from server import create_app, db
from routes.submissions import queue
from database.models import User, Submission, Run
from push import runSubmittedFile

# database problem name
PROB_NAME = 'The shortest path problem'
# database problem id (can be removed)
PROB_ID = 2
# folder name, used when creating directories needed
FOLDER_NAME = secure_filename(PROB_NAME).lower()
# zipfile name
ZIP_NAME = 'shortPath.zip'
TIMEOUT = 90


# helper to unzip submission
def unzip(filename, target):
    with zipfile.ZipFile(filename, 'r') as z:
        z.extractall(target)


class integrationSubmissionTest(TestCase):
    Testing = True

    def create_app(self):
        tmp = create_app(self)
        tmp.config['Testing'] = True
        tmp.config['SQLALCHEMY_DATABASE_URI'] = 'sqlite:///tests/testDBIntegr'
        return tmp

    def setUp(self):
        user = User(name='test', password='test')
        db.session.add(user)
        db.session.commit()
        self.upload_folder = self.app.config['UPLOAD_FOLDER'] + '/' + FOLDER_NAME + '/uploads/'
        self.user = user

    def tearDown(self):
        # Delete created rows
        user = User.query.filter_by(name='test').first()
        user_id = user.id
        user_name = user.name
        db.session.query(User).delete()
        db.session.query(Submission).delete()
        db.session.query(Run).delete()
        db.session.commit()
        db.session.remove()

        # Delete generated folder(s)
        rmtree(self.upload_folder + str(user_id) + '_' + user_name)

    def test_solution_run_check_output(self):
        token = create_jwt_token(self.app, self.user)
        with open(self.app.config['FLASK_DIR'] + "/source/tests/integration/" + ZIP_NAME, 'rb') as f:
            response = self.client.post("/api/upload/" + str(PROB_ID) + "/" + PROB_NAME + "/shortPath.zip",
                                        headers=[('Authorization', 'Bearer ' + token)],
                                        data=dict(file=f))
            self.assert200(response)
            obj = json.loads(response.data)
            user = obj['user']
            submissions = user['submissions']
            sub_id = submissions[0]['id']

            response = self.client.post("/api/enqueue/" + PROB_NAME + "/" + str(sub_id),
                                        headers=[('Authorization', 'Bearer ' + token)])
            self.assert200(response)
            self.assertTrue(queue.qsize() == 1)
            data = json.loads(response.data)
            self.assertTrue(data['user']['submissions'][0]['state'] == 'in queue')

            # start simulate push.py
            response = self.client.get("/api/dequeue", environ_base={'REMOTE_ADDR': '127.0.0.1'})
            self.assert200(response)
            self.assertTrue(queue.empty())
            self.assertTrue(queue.qsize() == 0)
            data = json.loads(response.data)
            # self.assertTrue(data['state'] == 'running')

            # if skip compile, then create directories needed
            compiledProblemFolder = self.app.config['FLASK_DIR'] + "/compiledSolutions/" + data['problemName']
            self.assertTrue(not os.path.exists(compiledProblemFolder))
            os.mkdir(compiledProblemFolder)
            compiledUserFolder = compiledProblemFolder + "/" + data['userName']
            self.assertTrue(not os.path.exists(compiledUserFolder))
            os.mkdir(compiledUserFolder)
            compiledSubFolder = compiledUserFolder + "/" + data['filename'][:-4]
            self.assertTrue(not os.path.exists(compiledSubFolder))
            # os.mkdir(compiledSubFolder)
            unzip(self.upload_folder + data['userName'] + "/" + data['filename'], compiledUserFolder)

            try:
                values = runSubmittedFile(data['problemName'], data['userName'], data['filename'])
            except subprocess.CalledProcessError as e:
                # values = '''{"isCorrect": false, "msg": "This should not happen! Mail the admins and tell them " +\
                #        " runScript.sh has failed. "}'''
                values = str(e.cmd) + " " + str(e.returncode) + " " + str(e.output)
            except:
                values = '''{"isCorrect":false, "msg": "Unhandled case on server. mail the admins"}'''
            rmtree(compiledProblemFolder)
            tmp = json.loads(values)
            tmp['submissionId'] = data['submissionId']
            tmp['runId'] = data['runId']
            response = self.client.put("/api/result",
                                       environ_base={'REMOTE_ADDR': '127.0.0.1'},
                                       data=json.dumps(tmp))
            self.assert200(response)
            # push done

            # retrieve database entry
            response = self.client.get("/api/submission/" + str(sub_id))
            while json.loads(response.data)['submission']['state'] == 'running':
                response = self.client.get("/api/submission/" + str(sub_id))
            self.assert200(response)
            data = json.loads(response.data)
            subm = data['submission']

            with open('test.txt', 'a+') as log:
                log.write("edp: " + str(subm['edp']) + ", energy: " + str(subm['energy']) +
                          ", time: " + str(subm['time']) + "\n")
