import json
from base64 import b64encode

from flask.ext.testing import TestCase
from sqlalchemy import and_

from cmb_utils.helpers import group_to_json, group_with_submission_to_json, my_group_to_json
from database.models import Group, GroupProblem, Member, Problem, User, Submission
from server import create_app, db


class groupTest(TestCase):
    Testing = True

    def create_app(self):
        tmp = create_app(self, push=False)
        tmp.config['SERVER_PORT'] = 1337
        tmp.config['Testing'] = True
        tmp.config['SQLALCHEMY_DATABASE_URI'] = "sqlite:///tests/testDB"
        return tmp

    def setUp(self):
        db.create_all()

    def tearDown(self):
        db.session.remove()
        db.drop_all()

    def getToken(self, name, password):
        headers = [('Content-Type', 'application/json')]
        data = {'name': name, 'password': b64encode(password)}
        json_data = json.dumps(data)
        json_data_length = len(json_data)
        headers.append(('Content-Length', json_data_length))
        login_response = self.client.post(
            '/auth/login',
            headers=headers,
            data=json_data)
        return login_response.json['token']

    def test_get_group(self):
        testPw = "password"
        user = User(name="Test", password=testPw)
        user2 = User(name="Test2", password="password")
        group = Group(name="Group", description="group")
        member = Member(user, group, "leader")
        db.session.add(user)
        db.session.add(user2)
        db.session.add(group)
        db.session.add(member)
        db.session.commit()

        token = self.getToken(user.name, testPw)
        headers = [('Authorization', 'Bearer ' + token)]
        response = self.client.get('/api/group/1', headers=headers)

        test_group = Group.query.filter_by(id=1).first()
        expected = dict(group=group_to_json(test_group))
        self.assert200(response)
        self.assertEquals(response.json, expected)

        headers = [('Authorization', 'Bearer ' + token)]
        response = self.client.get('/api/group/2', headers=headers)

        expected = dict(message="Group does not exist.")
        self.assert400(response)
        self.assertEquals(response.json, expected)

    def test_get_group_with_private_submissions(self):
        testPw = "password"
        user = User(name="Test", password=testPw)
        user2 = User(name="Test2", password=testPw)
        user3 = User(name="Test3", password=testPw)
        prob = Problem(name="Prob", description="test")
        sub = Submission("filename", 1, 2, "Uploaded")
        sub2 = Submission("filename2", 1, 1, "Uploaded", visible=False)
        group = Group(name="Group", description="group", public=True)
        member = Member(user, group, "leader")
        member2 = Member(user2, group, "member")
        grp_prob = GroupProblem(prob, group)
        db.session.add(user)
        db.session.add(user2)
        db.session.add(user3)
        db.session.add(group)
        db.session.add(prob)
        db.session.add(member)
        db.session.add(member2)
        db.session.add(grp_prob)
        db.session.add(sub)
        db.session.add(sub2)
        db.session.commit()

        # Test that leader get private submissions
        group_result = Group.query.filter_by(id=1).first()
        submissions = Submission.query.filter(and_(Submission.user_id.in_([1, 2]),
                                              Submission.problem_id.in_([1]))).all()
        expected = dict(group=group_with_submission_to_json(group_result, submissions))
        token = self.getToken(user.name, testPw)
        headers = [('Authorization', 'Bearer ' + token)]
        response = self.client.get('/api/group/1?with-submissions=1', headers=headers)
        self.assertEquals(response.json, expected)

        # Test that member get public submissions
        submissions = Submission.query.filter(and_(Submission.user_id.in_([1, 2]),
                                              Submission.problem_id.in_([1]))).filter_by(visible=True).all()
        expected = dict(group=group_with_submission_to_json(group_result, submissions))
        token = self.getToken(user2.name, testPw)
        headers = [('Authorization', 'Bearer ' + token)]
        response = self.client.get('/api/group/1?with-submissions=1', headers=headers)
        self.assertEquals(response.json, expected)

    def test_get_group_not_logged_in(self):
        testPw = "password"
        user = User(name="Test", password=testPw)
        user2 = User(name="Test2", password=testPw)
        user3 = User(name="Test3", password=testPw)
        prob = Problem(name="Prob", description="test")
        sub = Submission("filename", 1, 2, "Uploaded")
        sub2 = Submission("filename2", 1, 1, "Uploaded", visible=False)
        group = Group(name="Group", description="group", public=True)
        member = Member(user, group, "leader")
        member2 = Member(user2, group, "member")
        grp_prob = GroupProblem(prob, group)
        db.session.add(user)
        db.session.add(user2)
        db.session.add(user3)
        db.session.add(group)
        db.session.add(prob)
        db.session.add(member)
        db.session.add(member2)
        db.session.add(grp_prob)
        db.session.add(sub)
        db.session.add(sub2)
        db.session.commit()

        # Test that non-member does get group without submissions
        group_without = Group.query.filter_by(id=1, public=True).first()
        response = self.client.get('/api/group/1?with-submissions=1')
        test = dict(group=group_to_json(group_without))
        self.assertEquals(response.json, test)

    def test_toggle_group_public(self):
        testPw = "password"
        user = User(name="Test", password=testPw)
        group = Group("test", "test")
        member = Member(user, group, "leader")
        db.session.add(user)
        db.session.add(group)
        db.session.add(member)
        db.session.commit()
        assert not group.public

        token = self.getToken(user.name, testPw)
        headers = [('Authorization', 'Bearer ' + token)]
        response = self.client.put('/api/group/1', headers=headers)
        assert group.public

        headers = [('Authorization', 'Bearer ' + token)]
        response = self.client.put('/api/group/2', headers=headers)
        expected = dict(message="Group does not exist.")
        self.assert400(response)
        self.assertEquals(response.json, expected)

    def test_toggle_group_public_not_leader(self):
        testPw = "password"
        user = User(name="Test", password=testPw)
        group = Group("test", "test")
        member = Member(user, group, "member")
        db.session.add(user)
        db.session.add(group)
        db.session.add(member)
        db.session.commit()

        token = self.getToken(user.name, testPw)
        headers = [('Authorization', 'Bearer ' + token)]
        response = self.client.put('/api/group/1', headers=headers)
        expected = dict(message="Not Authorized to toggle group public")
        self.assert401(response)
        self.assertEquals(response.json, expected)

    def test_is_leader(self):
        testPw = "password"
        user = User(name="Test", password=testPw)
        group = Group(name="Group", description="group")
        member = Member(user, group, "leader")
        db.session.add(user)
        db.session.add(group)
        db.session.add(member)
        db.session.commit()

        token = self.getToken(user.name, testPw)
        expected = dict(message='Leader')
        headers = [('Authorization', 'Bearer ' + token)]
        response = self.client.get('/api/leader/1', headers=headers)
        self.assert200(response)
        self.assertEquals(response.json, expected)

        expected = dict(message='Group does not exist.')
        headers = [('Authorization', 'Bearer ' + token)]
        response = self.client.get('/api/leader/2', headers=headers)
        self.assert400(response)
        self.assertEquals(response.json, expected)

    def test_is_leader_not_leader(self):
        testPw = "password"
        user = User(name="Test", password=testPw)
        group = Group(name="Group", description="group")
        member = Member(user, group, "member")
        db.session.add(user)
        db.session.add(group)
        db.session.add(member)
        db.session.commit()

        token = self.getToken(user.name, testPw)
        expected = dict(message='Not Leader')
        headers = [('Authorization', 'Bearer ' + token)]
        response = self.client.get('/api/leader/1', headers=headers)
        self.assert200(response)
        self.assertEquals(response.json, expected)

    def test_get_public_groups(self):
        group = Group(name="Group", description="group")
        group2 = Group(name="Group1", description="group2")
        group3 = Group(name="Group2", description="group3")
        db.session.add(group)
        db.session.add(group2)
        db.session.add(group3)
        group2.public = True
        group3.public = True
        db.session.commit()

        response = self.client.get('/api/groups/public')
        self.assert200(response)
        resp = json.loads(response.data)
        self.assertEquals(len(resp['groups']), 2)

    def test_delete_group(self):
        testPw = "password"
        user = User(name="Test", password=testPw)
        group = Group(name="Group", description="group")
        member = Member(user, group, "leader")
        db.session.add(user)
        db.session.add(group)
        db.session.add(member)
        db.session.commit()

        token = self.getToken(user.name, testPw)
        headers = [('Authorization', 'Bearer ' + token)]
        response = self.client.delete('/api/group/1', headers=headers)

        test_group = Group.query.filter_by(id=1).first()
        expected = dict(message='Group 1 deleted.')
        self.assert200(response)
        self.assertEquals(response.json, expected)
        self.assertEquals(test_group, None)

        headers = [('Authorization', 'Bearer ' + token)]
        response = self.client.delete('/api/group/2', headers=headers)

        expected = dict(message='Group does not exist.')
        self.assert400(response)
        self.assertEquals(response.json, expected)

    def test_delete_group_not_leader(self):
        testPw = "password"
        user = User(name="Test", password=testPw)
        group = Group(name="Group", description="group")
        member = Member(user, group, "member")
        db.session.add(user)
        db.session.add(group)
        db.session.add(member)
        db.session.commit()

        token = self.getToken(user.name, testPw)
        headers = [('Authorization', 'Bearer ' + token)]
        response = self.client.delete('/api/group/1', headers=headers)

        test_group = Group.query.filter_by(id=1).first()
        expected = dict(message='Not Authorized to delete group 1')
        self.assert401(response)
        self.assertEquals(response.json, expected)
        self.assertEquals(test_group.name, "Group")

    def test_get_my_groups(self):
        testPw = "password"
        user = User(name="Test", password=testPw)
        user2 = User(name="Test2", password="password")
        group = Group(name="Group", description="group")
        group2 = Group(name="Group2", description="group2")
        member = Member(user, group, "leader")
        member2 = Member(user2, group2, "leader")
        member3 = Member(user, group2)
        db.session.add(user)
        db.session.add(user2)
        db.session.add(group)
        db.session.add(group2)
        db.session.add(member)
        db.session.add(member2)
        db.session.add(member3)
        db.session.commit()

        token = self.getToken(user.name, testPw)
        headers = [('Authorization', 'Bearer ' + token)]
        response = self.client.get('/api/mygroups', headers=headers)

        expected = dict(groups=map(my_group_to_json, [group, group2], [1, 1]))
        self.assert200(response)
        self.assertEquals(response.json, expected)

    def test_create_group_route(self):
        testPw = "password"
        user = User(name="Test", password=testPw)
        db.session.add(user)
        db.session.commit()

        token = self.getToken(user.name, testPw)
        headers = [('Content-Type', 'application/json')]
        data = {'name': 'group_name', 'description': 'test'}
        json_data = json.dumps(data)
        json_data_length = len(json_data)
        headers.append(('Content-Length', json_data_length))
        headers.append(('Authorization', 'Bearer ' + token))
        response = self.client.post('/api/group/create', headers=headers, data=json_data)

        test_group = Group.query.filter_by(id=1).first()
        expected = dict(group=group_to_json(test_group))
        self.assert200(response)
        self.assertEquals(response.json, expected)
        member = Member.query.filter_by(id=1).first()
        self.assertEquals(member.role, "leader")
        self.assertEquals(member.user, user)

    def test_create_group_without_name_route(self):
        testPw = "password"
        user = User(name="Test", password=testPw)
        db.session.add(user)
        db.session.commit()

        token = self.getToken(user.name, testPw)
        headers = [('Content-Type', 'application/json')]
        data = {'description': 'test'}
        json_data = json.dumps(data)
        json_data_length = len(json_data)
        headers.append(('Content-Length', json_data_length))
        headers.append(('Authorization', 'Bearer ' + token))
        response = self.client.post('/api/group/create', headers=headers, data=json_data)

        expected = dict(message='Group must have a name.')
        self.assert400(response)
        self.assertEquals(response.json, expected)

    def test_add_member_route(self):
        testPw = "password"
        user = User(name="Test", password=testPw)
        user2 = User(name="Test2", password="password")
        group = Group(name="Group", description="group")
        member = Member(user, group, "leader")
        db.session.add(user)
        db.session.add(user2)
        db.session.add(group)
        db.session.add(member)
        db.session.commit()

        token = self.getToken(user.name, testPw)
        headers = [('Authorization', 'Bearer ' + token)]
        response = self.client.post('/api/group/1/member/2', headers=headers)

        test_group = Group.query.filter_by(id=1).first()
        expected = dict(group=group_to_json(test_group))
        self.assert200(response)
        self.assertEquals(response.json, expected)

        member = Member.query.filter_by(id=2).first()
        self.assertEquals(member.role, "member")
        self.assertEquals(member.user, user2)
        self.assertEquals(member.group, group)

    def test_validate_not_user(self):
        testPw = "password"
        user = User(name="Test", password=testPw)
        group = Group(name="Group", description="group")
        member = Member(user, group, "leader")
        db.session.add(user)
        db.session.add(group)
        db.session.add(member)
        db.session.commit()

        token = self.getToken(user.name, testPw)
        headers = [('Authorization', 'Bearer ' + token)]
        response = self.client.post('/api/group/1/member/2', headers=headers)

        expected = dict(message='Object does not exist.')
        self.assert400(response)
        self.assertEquals(response.json, expected)

    def test_validate_not_group(self):
        testPw = "password"
        user = User(name="Test", password=testPw)
        user2 = User(name="Test2", password="password")
        db.session.add(user)
        db.session.add(user2)
        db.session.commit()

        token = self.getToken(user.name, testPw)
        headers = [('Authorization', 'Bearer ' + token)]
        response = self.client.post('/api/group/1/member/2', headers=headers)

        expected = dict(message='Group does not exist.')
        self.assert400(response)
        self.assertEquals(response.json, expected)

    def test_validate_not_leader(self):
        testPw = "password"
        user = User(name="Test", password=testPw)
        user2 = User(name="Test2", password="password")
        group = Group(name="Group", description="group")
        member = Member(user, group)
        db.session.add(user)
        db.session.add(user2)
        db.session.add(group)
        db.session.add(member)
        db.session.commit()

        token = self.getToken(user.name, testPw)
        headers = [('Authorization', 'Bearer ' + token)]
        response = self.client.post('/api/group/1/member/2', headers=headers)

        expected = dict(message='Must have group leader permissions.')
        self.assert401(response)
        self.assertEquals(response.json, expected)

    def test_validate_leader_not_in_group(self):
        testPw = "password"
        user = User(name="Test", password=testPw)
        user2 = User(name="Test2", password="password")
        group = Group(name="Group", description="group")
        db.session.add(user)
        db.session.add(user2)
        db.session.add(group)
        db.session.commit()

        token = self.getToken(user.name, testPw)
        headers = [('Authorization', 'Bearer ' + token)]
        response = self.client.post('/api/group/1/member/2', headers=headers)

        expected = dict(message='Must have group leader permissions.')
        self.assert401(response)
        self.assertEquals(response.json, expected)

    def test_delete_member_route(self):
        testPw = "password"
        user = User(name="Test", password=testPw)
        user2 = User(name="Test2", password="password")
        group = Group(name="Group", description="group")
        member = Member(user, group, "leader")
        member2 = Member(user2, group)
        db.session.add(user)
        db.session.add(user2)
        db.session.add(group)
        db.session.add(member)
        db.session.add(member2)
        db.session.commit()

        token = self.getToken(user.name, testPw)
        headers = [('Authorization', 'Bearer ' + token)]
        response = self.client.delete('/api/group/1/member/2', headers=headers)

        test_group = Group.query.filter_by(id=1).first()
        expected = dict(group=group_to_json(test_group))
        self.assert200(response)
        self.assertEquals(response.json, expected)

        member = Member.query.filter_by(id=2).first()
        self.assertEquals(member, None)

    def test_add_problem_route(self):
        testPw = "password"
        user = User(name="Test", password=testPw)
        prob = Problem(name="Prob", description="test")
        group = Group(name="Group", description="group")
        member = Member(user, group, "leader")
        db.session.add(user)
        db.session.add(group)
        db.session.add(prob)
        db.session.add(member)
        db.session.commit()

        token = self.getToken(user.name, testPw)

        headers = [('Authorization', 'Bearer ' + token)]
        response = self.client.post('/api/group/1/problem/1', headers=headers)

        test_group = Group.query.filter_by(id=1).first()
        expected = dict(group=group_to_json(test_group))
        self.assert200(response)
        self.assertEquals(response.json, expected)

        grp_prob = GroupProblem.query.filter_by(id=1).first()
        self.assertEquals(grp_prob.problem, prob)
        self.assertEquals(grp_prob.group, group)

    def test_delete_problem_route(self):
        testPw = "password"
        user = User(name="Test", password=testPw)
        prob = Problem(name="Prob", description="test")
        group = Group(name="Group", description="group")
        member = Member(user, group, "leader")
        grp_prob = GroupProblem(prob, group)
        db.session.add(user)
        db.session.add(group)
        db.session.add(prob)
        db.session.add(member)
        db.session.add(grp_prob)
        db.session.commit()

        token = self.getToken(user.name, testPw)
        headers = [('Authorization', 'Bearer ' + token)]
        response = self.client.delete('/api/group/1/problem/1', headers=headers)

        test_group = Group.query.filter_by(id=1).first()
        expected = dict(group=group_to_json(test_group))
        self.assert200(response)
        self.assertEquals(response.json, expected)

        deleted_grp_prob = GroupProblem.query.filter_by(id=1).first()
        self.assertEquals(deleted_grp_prob, None)

    def test_leave_group_route(self):
        testPw = "password"
        user = User(name="Test", password=testPw)
        group = Group(name="Group", description="group")
        member = Member(user, group, "member")
        db.session.add(user)
        db.session.add(group)
        db.session.add(member)
        db.session.commit()

        token = self.getToken(user.name, testPw)
        headers = [('Authorization', 'Bearer ' + token)]
        response = self.client.delete('/api/group/leave/1', headers=headers)

        test_member = Member.query.filter_by(id=1).first()
        self.assert200(response)
        self.assertEquals(test_member, None)

    def test_leave_group_route_not_member(self):
        testPw = "password"
        user = User(name="Test", password=testPw)
        group = Group(name="Group", description="group")
        db.session.add(user)
        db.session.add(group)
        db.session.commit()

        token = self.getToken(user.name, testPw)
        headers = [('Authorization', 'Bearer ' + token)]
        response = self.client.delete('/api/group/leave/1', headers=headers)

        expected = dict(message='Not a member.')
        self.assert400(response)
        self.assertEquals(response.json, expected)

    def test_leave_group_route_leader(self):
        testPw = "password"
        user = User(name="Test", password=testPw)
        group = Group(name="Group", description="group")
        member = Member(user, group, "leader")
        db.session.add(user)
        db.session.add(group)
        db.session.add(member)
        db.session.commit()

        token = self.getToken(user.name, testPw)

        headers = [('Authorization', 'Bearer ' + token)]
        response = self.client.delete('/api/group/leave/1', headers=headers)

        expected = dict(message='Leader cannot leave group.')
        self.assert400(response)
        self.assertEquals(response.json, expected)

    def test_join_group_route(self):
        testPw = "password"
        user = User(name="Test", password=testPw)
        group = Group(name="Group", description="group")
        db.session.add(user)
        db.session.add(group)
        db.session.commit()

        token = self.getToken(user.name, testPw)
        headers = [('Authorization', 'Bearer ' + token)]
        response = self.client.post('/api/group/join/1', headers=headers)
        self.assert400(response)
        expected = dict(message='Group is not public.')
        self.assertEquals(response.json, expected)

        group.public = True
        db.session.commit()
        headers = [('Authorization', 'Bearer ' + token)]
        response = self.client.post('/api/group/join/1', headers=headers)
        test_member = Member.query.filter_by(id=1).first()
        self.assert200(response)
        self.assertEquals(test_member.user, user)

        headers = [('Authorization', 'Bearer ' + token)]
        response = self.client.post('/api/group/join/2', headers=headers)
        self.assert400(response)
        expected = dict(message='Group does not exist.')
        self.assertEquals(response.json, expected)

    def test_join_group_already_member(self):
        testPw = "password"
        user = User(name="Test", password=testPw)
        group = Group(name="Group", description="group")
        member = Member(user, group, "leader")
        db.session.add(user)
        db.session.add(group)
        db.session.add(member)
        db.session.commit()

        token = self.getToken(user.name, testPw)

        headers = [('Authorization', 'Bearer ' + token)]
        response = self.client.post('/api/group/join/1', headers=headers)
        self.assert400(response)
        expected = dict(message='Already a member.')
        self.assertEquals(response.json, expected)
