from flask.ext.testing import TestCase
import mock

import cmb_utils.helpers
from database.models import (Group, GroupProblem, Member, Problem, Submission,
                             User)
from server import create_app, db


class serverTest(TestCase):
    Testing = True

    def create_app(self):
        tmp = create_app(self, push=False)
        # tmp.config['SERVER_PORT']=1337
        tmp.config['Testing'] = True
        tmp.config['SQLALCHEMY_DATABASE_URI'] = "sqlite:///tests/testDB"
        return tmp

    def setUp(self):
        db.create_all()

    def tearDown(self):
        db.session.remove()
        db.drop_all()

    def test_user_to_json(self):
        user = User(name="Test", password="test")
        self.assertEquals(cmb_utils.helpers.user_to_json(user), dict(id=None, name="Test", submissions=[]))

    def test_me_to_json(self):
        user = User(name="Test", password="test", email="test@example.com")
        self.assertEquals(cmb_utils.helpers.me_to_json(user),
                          dict(id=None, name="Test", email="test@example.com", submissions=[]))

    def test_problem_to_json(self):
        problem = Problem("Test", "test")
        self.assertEquals(cmb_utils.helpers.problem_to_json(problem),
                          dict(id=None, name="Test", allow_profiling=True, description="test",
                               submissions=[], goodness=None))

    def test_submission_to_json(self):
        problem = Problem("Test", "test")
        user = User(name="Test", password="test")
        db.session.add(problem)
        db.session.add(user)
        db.session.commit()
        submission = Submission("filename", 1, 1, "Uploaded")
        db.session.add(submission)
        db.session.commit()
        self.assertEquals(cmb_utils.helpers.submission_to_json(submission),
                          dict(id=1, problem_id=1, profiling_data=None, user_id=1, user_name=u"Test",
                               name=u"filename", time=None, energy=None, has_profiling_data=False,
                               edp=None, state=u"Uploaded", detailed_state=None, msg=None, goodness=None,
                               visible=True))

    def test_construct_submission_folder_path(self):
        problem = Problem("test_problem", "test")
        user = User(name="test", password="test")
        sub = Submission("test.zip", 1, 1, "Uploaded")

        result = cmb_utils.helpers.construct_submission_folder_path(user, problem, sub)
        expected = "/test_problem/uploads/None_test/test.zip"
        self.assertEquals(result, self.app.config['UPLOAD_FOLDER'] + expected)

    @mock.patch('cmb_utils.helpers.os')
    @mock.patch('cmb_utils.helpers.os.path')
    @mock.patch('cmb_utils.helpers.secure_filename', return_value="secure")
    def test_delete_submission_files(self, mock_secure, mock_path, mock_os):
        problem = Problem("test_problem", "test")
        user = User(name="test", password="test")
        sub = Submission("test.zip", 1, 1, "Uploaded")
        mock_path.exists.return_value = True
        cmb_utils.helpers.delete_submission_files(user, problem, sub)
        # correct path covered in another test
        mock_os.remove.assert_called_once()

    def test_create_and_parse_token(self):
        user = User(name="Test", password="test")
        db.session.add(user)
        db.session.commit()
        test_user = User.query.filter_by(id=1).first()
        token = cmb_utils.helpers.create_jwt_token(self.app, test_user)
        payload = cmb_utils.helpers.parse_token(self.app, token)
        self.assertEquals(payload['sub'], test_user.id)

    def test_allowed_file(self):
        correct = "test.zip"
        tar = "test.tar.gz"
        cpp = "test.cpp"
        assert cmb_utils.helpers.allowed_file(correct)
        assert not cmb_utils.helpers.allowed_file(tar)
        assert not cmb_utils.helpers.allowed_file(cpp)

    def test_filter_submission_by_problem(self):
        user = User("Test", "test")
        user2 = User("Test2", "test2")
        prob = Problem("Test", "test")
        prob2 = Problem("Test2", "test2")
        sub = Submission("Sub", 1, 1, "Uploaded")
        sub2 = Submission("Sub2", 2, 2, "Uploaded")

        db.session.add(user)
        db.session.add(user2)
        db.session.add(prob)
        db.session.add(prob2)
        db.session.add(sub)
        db.session.add(sub2)
        db.session.commit()

        submissions = Submission.query.filter_by(visible=True).all()

        filter1 = cmb_utils.helpers.filter_submissions_by_problem(submissions, 1)
        filter2 = cmb_utils.helpers.filter_submissions_by_problem(submissions, 2)

        self.assertEquals(cmb_utils.helpers.submission_to_json(filter1[0]),
                          dict(id=1, problem_id=1, user_id=1, user_name=u"Test", name=u"Sub", time=None, energy=None,
                               edp=None, state=u"Uploaded", detailed_state=None, msg=None, goodness=None, visible=True,
                               has_profiling_data=False, profiling_data=None))

        self.assertEquals(cmb_utils.helpers.submission_to_json(filter2[0]),
                          dict(id=2, problem_id=2, user_id=2, user_name=u"Test2", name=u"Sub2", time=None, energy=None,
                               edp=None, state=u"Uploaded", detailed_state=None, msg=None, goodness=None, visible=True,
                               has_profiling_data=False, profiling_data=None))

    def test_member_to_json(self):
        group = Group("Group_test", "Testing group")
        user = User("Test", "test")
        db.session.add(group)
        db.session.add(user)
        member = Member(user, group)
        db.session.add(member)
        db.session.commit()
        new_member = Member.query.filter_by(id=1).first()
        self.assertEquals(cmb_utils.helpers.member_to_json(new_member),
                          dict(id=1,
                               user_id=member.user_id,
                               username=member.user.name,
                               group_id=member.group_id,
                               role="member"))

    def test_group_problem_to_json(self):
        group = Group("Group_test", "Testing group")
        prob = Problem("Test", "test")
        db.session.add(group)
        db.session.add(prob)
        group_problem = GroupProblem(prob, group)
        db.session.add(group_problem)
        db.session.commit()
        new_group_problem = GroupProblem.query.filter_by(id=1).first()
        self.assertEquals(cmb_utils.helpers.group_problem_to_json(new_group_problem),
                          dict(id=1,
                               problem_id=new_group_problem.problem_id,
                               problem_name=new_group_problem.problem.name,
                               group_id=new_group_problem.group_id))

    def test_group_to_json(self):
        group = Group("Group_test", "Testing group")
        user = User("Test", "test")
        user2 = User("Test2", "test2")
        prob = Problem("Test", "test")
        prob2 = Problem("Test2", "test2")
        db.session.add(group)
        db.session.add(user)
        db.session.add(user2)
        db.session.add(prob)
        db.session.add(prob2)
        member = Member(user, group)
        member2 = Member(user2, group, "leader")
        group_problem = GroupProblem(prob, group)
        group_problem2 = GroupProblem(prob2, group)
        db.session.add(member)
        db.session.add(member2)
        db.session.add(group_problem)
        db.session.add(group_problem2)
        db.session.commit()
        new_group = Group.query.filter_by(id=1).first()
        self.assertEquals(cmb_utils.helpers.group_to_json(new_group),
                          dict(id=1,
                               name="Group_test",
                               description="Testing group",
                               members=[cmb_utils.helpers.member_to_json(member),
                                        cmb_utils.helpers.member_to_json(member2)],
                               problems=[cmb_utils.helpers.group_problem_to_json(group_problem),
                                         cmb_utils.helpers.group_problem_to_json(group_problem2)],
                               public=False))

    def test_group_with_submissions_to_json(self):
        group = Group("Group_test", "Testing group")
        user = User("Test", "test")
        user2 = User("Test2", "test2")
        prob = Problem("Test", "test")
        prob2 = Problem("Test2", "test2")
        sub = Submission("Sub", 1, 1)
        sub2 = Submission("Sub2", 2, 2)
        member = Member(user, group)
        member2 = Member(user2, group, "leader")
        group_problem = GroupProblem(prob, group)
        group_problem2 = GroupProblem(prob2, group)

        db.session.add(group)
        db.session.add(user)
        db.session.add(user2)
        db.session.add(prob)
        db.session.add(prob2)
        db.session.add(sub)
        db.session.add(sub2)
        db.session.add(member)
        db.session.add(member2)
        db.session.add(group_problem)
        db.session.add(group_problem2)
        db.session.commit()

        group = Group.query.filter_by(id=1).first()
        submissions = Submission.query.all()

        group_problem.submissions = cmb_utils.helpers.filter_submissions_by_problem(submissions, 1)
        group_problem2.submissions = cmb_utils.helpers.filter_submissions_by_problem(submissions, 2)

        result = cmb_utils.helpers.group_with_submission_to_json(group, submissions)
        expected = dict(id=1,
                        name="Group_test",
                        description="Testing group",
                        members=[cmb_utils.helpers.member_to_json(member),
                                 cmb_utils.helpers.member_to_json(member2)],
                        problems=[cmb_utils.helpers.group_problem_to_json(group_problem),
                                  cmb_utils.helpers.group_problem_to_json(group_problem2)],
                        public=False)

        self.assertEquals(result, expected)
        self.assertEquals(cmb_utils.helpers.submission_to_json(group_problem.submissions[0]),
                          cmb_utils.helpers.submission_to_json(sub))
        self.assertEquals(cmb_utils.helpers.submission_to_json(group_problem2.submissions[0]),
                          cmb_utils.helpers.submission_to_json(sub2))
