from datetime import datetime

from flask.ext.testing import TestCase

from database.models import (Group, GroupProblem, Member, Problem, Run,
                             Submission, User)
from server import create_app, db


class ModelsTest(TestCase):
    Testing = True

    def create_app(self):
        tmp = create_app(self, push=False)
        tmp.config['SQLALCHEMY_DATABASE_URI'] = "sqlite:///tests/testDB"
        # tmp.config['SERVER_PORT']=1337
        return tmp

    def setUp(self):
        db.create_all()

    def tearDown(self):
        db.session.remove()
        db.drop_all()

    def test_user_creation_and_create_and_check_password(self):
        user = User(name="Test", password="test")
        db.session.add(user)
        db.session.commit()
        test_user = User.query.filter_by(name="Test").first()
        assert test_user.password != "test"
        assert test_user.check_password("test")

    def test_problem_creation(self):
        problem = Problem("Test", "test")
        db.session.add(problem)
        db.session.commit()
        test_problem = Problem.query.filter_by(name="Test").first()
        Problem.query.all()
        assert test_problem.description == "test"

    def test_submission_creation(self):
        problem = Problem("Test", "test")
        problem2 = Problem("Test2", "test2")
        db.session.add(problem)
        db.session.add(problem2)
        user = User(name="Test", password="test")
        user2 = User(name="Test2", password="test2")
        db.session.add(user)
        db.session.add(user2)
        db.session.commit()
        sub = Submission("filename", 1, 1, "Uploaded")
        db.session.add(sub)
        db.session.commit()
        test_sub = Submission.query.filter_by(name="filename").first()
        assert test_sub.problem == problem
        assert test_sub.problem != problem2
        assert test_sub.user == user
        assert test_sub.user != user2

    def test_run_creation(self):
        submission = Submission("Test", "test")
        db.session.add(submission)
        db.session.commit()
        run = Run(submission_id=1)
        db.session.add(run)
        db.session.commit()
        test_run = Run.query.filter_by(id=1).first()
        assert test_run.submission == submission

    def test_submission_run_deletion(self):
        submission = Submission("Test", "test")
        run1 = Run(submission_id=1)
        run2 = Run(submission_id=1)
        run3 = Run(submission_id=1)
        run4 = Run(submission_id=1)
        run5 = Run(submission_id=1)
        db.session.add(submission)
        db.session.add(run1)
        db.session.add(run2)
        db.session.add(run3)
        db.session.add(run4)
        db.session.add(run5)
        db.session.commit()

        num_submissions = len(Submission.query.all())
        num_runs = len(Run.query.all())

        assert num_submissions == 1
        assert num_runs == 5

        db.session.delete(run5)
        db.session.commit()

        num_submissions = len(Submission.query.all())
        num_runs = len(Run.query.all())

        assert num_submissions == 1
        assert num_runs == 4

        db.session.delete(submission)
        db.session.commit()

        num_submissions = len(Submission.query.all())
        num_runs = len(Run.query.all())

        assert num_submissions == 0
        assert num_runs == 0

    def test_group_creation(self):
        group = Group("Group_test", "Testing group")
        db.session.add(group)
        db.session.commit()
        new_group = Group.query.filter_by(id=1).first()
        assert new_group.name == group.name

    def test_add_problem_to_group(self):
        group = Group("Group_test", "Testing group")
        problem = Problem("Test", "test")
        time = datetime.now()
        group_problem = GroupProblem(problem, group, time)
        db.session.add(group)
        db.session.add(problem)
        db.session.add(group_problem)
        self.assertEquals(group.problems[0].problem, problem)
        self.assertEquals(group.problems[0].deadline, time)
        assert group.problems[0].visible is not None

    def test_add_user_to_group(self):
        group = Group("Group_test", "Testing group")
        user = User("Test", "test")
        member = Member(user, group)
        db.session.add(group)
        db.session.add(user)
        db.session.add(member)
        self.assertEquals(group.members[0].user, user)
        self.assertEquals(user.groups[0].group, group)
        self.assertEquals(user.groups[0].role, "member")
        user2 = User("Test2", "test2")
        member = Member(user2, group, "leader")
        self.assertEquals(user2.groups[0].role, "leader")
        self.assertEquals(group.members[0].user, user)
        self.assertEquals(group.members[1].user, user2)
        self.assertEquals(group.members[1].role, "leader")
