import json
from StringIO import StringIO

from flask.ext.testing import TestCase
from mock import patch
from werkzeug import secure_filename

from cmb_utils.compilation_check import CompilationError
from cmb_utils.helpers import (create_jwt_token, submission_to_json,
                               user_to_json)
from database.models import Problem, Run, Submission, User
from routes.submissions import queue
from server import create_app, db


import cmb_utils


def create_result_obj(sub, run):
    obj = dict(submissionId=sub.id, runId=run.id, isCorrect=True, time=13, energy=20, power=0, goodness=None)
    return obj


class submissionsTest(TestCase):
    Testing = True

    def create_app(self):
        tmp = create_app(self, push=False)
        tmp.config['SERVER_PORT'] = 1337
        tmp.config['Testing'] = True
        tmp.config['SQLALCHEMY_DATABASE_URI'] = "sqlite:///tests/testDB"
        return tmp

    def setUp(self):
        db.create_all()
        problem = Problem("testName", "testDescription")
        db.session.add(problem)
        user = User("username", "password")
        db.session.add(user)
        sub = Submission("subName", 1, 1)
        db.session.add(sub)
        run = Run()
        db.session.add(run)
        db.session.commit()

    def tearDown(self):
        db.session.remove()
        db.drop_all()

    def test_get_submission_route(self):
        test_sub = Submission.query.filter_by(id=1).first()
        response = self.client.get("/api/submission/1")
        expected = dict(submission=submission_to_json(test_sub))

        self.assert200(response)
        self.assertEquals(response.json, expected)

    def test_upload_submission_route(self):
        user = User.query.filter_by(id=1).first()
        token = create_jwt_token(self.app, user)
        with patch('routes.submissions.saveFile') as MockClass:
            response = self.client.post("/api/upload/1/testName/test_sub.zip",
                                        headers=[('Authorization', 'Bearer ' + token)],
                                        data=dict(file=(StringIO('content'), 'test.zip')))
            self.assert200(response)
            expected = dict(user=user_to_json(user))
            self.assertEquals(response.json, expected)
            MockClass.assert_called_once()

            response = self.client.post("/api/upload/1/testName/test_sub.rar",
                                        headers=[('Authorization', 'Bearer ' + token)],
                                        data=dict(file=(StringIO('content'), 'test.zip')))
            self.assert400(response)

    def check_compilation_mock(self): return None

    @patch('routes.submissions.CompilationCheck.run', check_compilation_mock)
    def test_compile_route_success(self):
        user = User.query.filter_by(id=1).first()
        problem = Problem.query.filter_by(id=1).first()
        sub = Submission.query.filter_by(id=1).first()
        token = create_jwt_token(self.app, user)

        with patch.object(cmb_utils.compilation_check.CompilationCheck, 'run') as MockClass:
            MockClass.return_value = None
            response = self.client.post("/api/compile/" + problem.name + "/" + str(sub.id),
                                        headers=[('Authorization', 'Bearer ' + token)])
            self.assert200(response)
            self.assertEquals(sub.state, "compiled")
            expected = dict(user=user_to_json(user))
            self.assertEquals(response.json, expected)

    def test_compile_route_failure(self):
        user = User.query.filter_by(id=1).first()
        problem = Problem.query.filter_by(id=1).first()
        sub = Submission.query.filter_by(id=1).first()
        token = create_jwt_token(self.app, user)
        with patch.object(cmb_utils.compilation_check.CompilationCheck, 'run') as MockClass:
            MockClass.side_effect = CompilationError('Compilation failed')
            response = self.client.post("/api/compile/" + problem.name + "/" + str(sub.id),
                                        headers=[('Authorization', 'Bearer ' + token)])
            expected = dict(user=user_to_json(user))
            self.assert200(response)
            self.assertEquals(sub.state, "failed")
            self.assertEquals(sub.msg, "Compilation failed")
            self.assertEquals(response.json, expected)

    def test_result_route(self):
        sub = Submission.query.filter_by(id=1).first()
        run = Run.query.filter_by(id=1).first()
        assert sub.state is None
        assert sub.time is None
        assert sub.energy is None
        assert sub.edp is None

        obj = create_result_obj(sub, run)
        response = self.client.put("/api/result", data=json.dumps(obj), environ_base={'REMOTE_ADDR': '127.0.0.1'})
        self.assert200(response)
        assert sub.state == "finished"
        assert sub.time == 13.0
        assert sub.energy == 20.0
        assert sub.edp == 260.0

    def test_result_route_with_empty_goodness(self):
        sub = Submission.query.filter_by(id=1).first()
        run = Run.query.filter_by(id=1).first()
        assert sub.goodness_value is None

        obj = create_result_obj(sub, run)
        response = self.client.put("/api/result", data=json.dumps(obj), environ_base={'REMOTE_ADDR': '127.0.0.1'})
        self.assert200(response)
        assert sub.goodness_value is None

    def test_result_route_with_goodness(self):
        sub = Submission.query.filter_by(id=1).first()
        run = Run.query.filter_by(id=1).first()
        assert sub.goodness_value is None

        obj = create_result_obj(sub, run)
        obj['goodness'] = "1337"
        response = self.client.put("/api/result", data=json.dumps(obj), environ_base={'REMOTE_ADDR': '127.0.0.1'})
        self.assert200(response)
        assert sub.goodness_value == 1337.0

    def test_result_route_with_isCorrect_false(self):
        sub = Submission.query.filter_by(id=1).first()
        run = Run.query.filter_by(id=1).first()
        assert sub.state is None
        assert sub.msg is None
        obj = create_result_obj(sub, run)
        obj['isCorrect'] = False
        obj['msg'] = "Something went wrong"
        response = self.client.put("/api/result", data=json.dumps(obj), environ_base={'REMOTE_ADDR': '127.0.0.1'})
        self.assert200(response)
        assert sub.state == "failed"
        assert sub.msg == "Something went wrong"

    def test_toggle_visible_route(self):
        user = User.query.filter_by(id=1).first()
        sub = Submission.query.filter_by(id=1).first()
        assert sub.visible is True
        token = create_jwt_token(self.app, user)
        response = self.client.put("/api/submission/1", None, headers=[('Authorization', 'Bearer ' + token)])
        self.assert200(response)
        assert sub.visible is False

    def test_enqueue_route(self):
        user = User.query.filter_by(id=1).first()
        problem = Problem.query.filter_by(id=1).first()
        sub = Submission.query.filter_by(id=1).first()
        token = create_jwt_token(self.app, user)
        self.assertEquals(sub.state, None)
        response = self.client.post("/api/enqueue/" + problem.name + "/" + str(sub.id) + "/" + "False",
                                    headers=[('Authorization', 'Bearer ' + token)])
        self.assert200(response)
        self.assertEquals(sub.state, "in queue")
        self.assertEquals(queue.qsize(), 1)

        queue.get()
        self.assertEquals(queue.qsize(), 0)

    def test_dequeue_route(self):
        user = User.query.filter_by(id=1).first()
        problem = Problem.query.filter_by(id=1).first()
        sub = Submission.query.filter_by(id=1).first()
        run = Run.query.filter_by(id=1).first()
        obj = {'filename': sub.name, 'problemName': problem.name,
               'userName': str(user.id) + "_" + secure_filename(user.name).lower(), 'submissionId': sub.id,
               'runId': run.id, 'profiling': False}
        queue.put(obj)
        self.assertEquals(queue.qsize(), 1)
        self.assertEquals(sub.state, None)

        response = self.client.get("/api/dequeue", environ_base={'REMOTE_ADDR': '127.0.0.1'})
        self.assert200(response)
        self.assertEquals(sub.state, "running")
        self.assertEquals(response.json, obj)
        self.assertEquals(queue.qsize(), 0)

    def test_delete_submission_route(self):
        user = User.query.filter_by(id=1).first()
        token = create_jwt_token(self.app, user)
        response = self.client.delete("/api/submission/2")
        self.assert401(response)

        response = self.client.delete("/api/submission/2",
                                      headers=[('Authorization', 'Bearer ' + token)])
        self.assert400(response)
        with patch('routes.submissions.delete_submission_files') as MockClass:
            response = self.client.delete("/api/submission/1",
                                          headers=[('Authorization', 'Bearer ' + token)])
            MockClass.assert_called_once()
            self.assert200(response)
            self.assertEquals(response.json, dict(message="Submission 1 deleted."))
