import json
from base64 import b64encode

from flask.ext.testing import TestCase

from cmb_utils.helpers import me_to_json, user_to_json
from cmb_utils.mail import mail
from database.models import User
from server import create_app, db


class problemsTest(TestCase):
    Testing = True

    def create_app(self):
        tmp = create_app(self, push=False)
        # tmp.config['SERVER_PORT']=1337
        tmp.config['Testing'] = True
        tmp.config['SQLALCHEMY_DATABASE_URI'] = "sqlite:///tests/testDB"
        tmp.config.update(TESTING=True)
        mail.init_app(tmp)
        return tmp

    def setUp(self):
        db.create_all()

    def tearDown(self):
        db.session.remove()
        db.drop_all()

    def test_signup(self):
        with mail.record_messages() as outbox:
            headers = [('Content-Type', 'application/json')]
            data = {'email': 'ntnu.cmb@gmail.com', 'name': 'Test', 'password': b64encode("password")}
            json_data = json.dumps(data)
            json_data_length = len(json_data)
            headers.append(('Content-Length', json_data_length))
            response = self.client.post('/auth/signup', headers=headers, data=json_data)
            self.assertEquals(response.status_code, 200)
            assert response.json['token'] is not None
            assert len(outbox) == 1
            assert outbox[0].subject == "Welcome to Climbing Mont Blanc"
            user = User.query.filter_by(name="Test").first()
            assert user.email == 'ntnu.cmb@gmail.com'

    def test_signup_email_exist(self):
        user = User(name="Name", email="test@test.no", password="password")
        db.session.add(user)
        db.session.commit()
        headers = [('Content-Type', 'application/json')]
        data = {'email': 'test@test.no', 'name': 'Test', 'password': b64encode("password")}
        json_data = json.dumps(data)
        json_data_length = len(json_data)
        headers.append(('Content-Length', json_data_length))
        response = self.client.post('/auth/signup', headers=headers, data=json_data)
        self.assertEquals(response.status_code, 400)
        self.assertEquals(response.json['message'], "User with email already exist!")

    def test_signup_username_exist(self):
        user = User(name="Test", email="name@test.no", password="password")
        db.session.add(user)
        db.session.commit()
        headers = [('Content-Type', 'application/json')]
        data = {'email': 'test@test.no', 'name': 'Test', 'password': b64encode("password")}
        json_data = json.dumps(data)
        json_data_length = len(json_data)
        headers.append(('Content-Length', json_data_length))
        response = self.client.post('/auth/signup', headers=headers, data=json_data)
        self.assertEquals(response.status_code, 400)
        self.assertEquals(response.json['message'], "Username already exist!")

    def test_login_and_fetch_user_data(self):
        user = User(name="Test", password="password")
        db.session.add(user)
        db.session.commit()

        response = self.client.get('/api/me')
        self.assertEquals(response.status_code, 401)

        headers = [('Content-Type', 'application/json')]
        data = {'name': 'Test', 'password': b64encode("password")}
        json_data = json.dumps(data)
        json_data_length = len(json_data)
        headers.append(('Content-Length', json_data_length))
        login_response = self.client.post('/auth/login', headers=headers, data=json_data)
        self.assertEquals(login_response.status_code, 200)
        assert login_response.json['token'] is not None

        test_user = User.query.filter_by(name="Test").first()
        me_response = self.client.get('/api/me', headers=[('Authorization', 'Bearer ' + login_response.json['token'])])
        self.assertEquals(me_response.status_code, 200)
        self.assertEquals(me_response.json, dict(user=me_to_json(test_user)))

    def test_get_user(self):
        user = User(name="Test", password="password")
        db.session.add(user)
        db.session.commit()
        response = self.client.get('/api/user/1')
        test_user = User.query.filter_by(name="Test").first()
        self.assertEquals(response.status_code, 200)
        self.assertEquals(response.json, dict(user=user_to_json(test_user)))

    def test_get_users(self):
        user = User(name="Test", password="password")
        db.session.add(user)
        user2 = User(name="Test2", password="password")
        db.session.add(user2)
        db.session.commit()
        response = self.client.get('/api/users')
        test_user = User.query.filter_by(name="Test").first()
        test_user2 = User.query.filter_by(name="Test2").first()
        self.assertEquals(response.status_code, 200)
        self.assertEquals(response.json, dict(users=[user_to_json(test_user), user_to_json(test_user2)]))

    def test_update_email(self):
        user = User(name="Test", password="password")
        db.session.add(user)
        db.session.commit()
        test_user = User.query.filter_by(name="Test").first()

        headers = [('Content-Type', 'application/json')]
        data = {'name': 'Test', 'password': b64encode("password")}
        json_data = json.dumps(data)
        json_data_length = len(json_data)
        headers.append(('Content-Length', json_data_length))
        login_response = self.client.post('/auth/login', headers=headers, data=json_data)
        token = login_response.json['token']

        headers = [('Content-Type', 'application/json')]
        data = {'type': 'email', 'email': 'new_email@test.no', 'password': b64encode("password")}
        json_data = json.dumps(data)
        json_data_length = len(json_data)
        headers.append(('Content-Length', json_data_length))
        headers.append(('Authorization', 'Bearer ' + token))
        with mail.record_messages() as outbox:
            response = self.client.put('/api/me', headers=headers, data=json_data)
            self.assertEquals(response.status_code, 200)
            self.assertEquals(response.json, dict(user=me_to_json(test_user)))
            self.assertEquals(len(outbox), 1)
            self.assertEquals(outbox[0].subject, "Climbing Mont Blanc: Email Changed")

    def test_update_email_with_existing_email(self):
        user = User(name="Test", password="password")
        db.session.add(user)
        user2 = User(name="Test", email="test@test.no", password="password2")
        db.session.add(user2)
        db.session.commit()

        headers = [('Content-Type', 'application/json')]
        data = {'name': 'Test', 'password': b64encode("password")}
        json_data = json.dumps(data)
        json_data_length = len(json_data)
        headers.append(('Content-Length', json_data_length))
        login_response = self.client.post('/auth/login', headers=headers, data=json_data)
        token = login_response.json['token']

        headers = [('Content-Type', 'application/json')]
        data = {'type': 'email', 'email': 'test@test.no', 'password': b64encode("password")}
        json_data = json.dumps(data)
        json_data_length = len(json_data)
        headers.append(('Content-Length', json_data_length))
        headers.append(('Authorization', 'Bearer ' + token))

        response = self.client.put('/api/me', headers=headers, data=json_data)
        self.assert400(response)

    def test_update_password(self):
        user = User(name="Test", password="password")
        db.session.add(user)
        db.session.commit()
        test_user = User.query.filter_by(name="Test").first()

        headers = [('Content-Type', 'application/json')]
        data = {'name': 'Test', 'password': b64encode("password")}
        json_data = json.dumps(data)
        json_data_length = len(json_data)
        headers.append(('Content-Length', json_data_length))
        login_response = self.client.post('/auth/login', headers=headers, data=json_data)
        token = login_response.json['token']

        headers = [('Content-Type', 'application/json')]
        data = {'type': 'password', 'new_password': b64encode('new_password'), 'password': b64encode("password")}
        json_data = json.dumps(data)
        json_data_length = len(json_data)
        headers.append(('Content-Length', json_data_length))
        headers.append(('Authorization', 'Bearer ' + token))
        with mail.record_messages() as outbox:
            response = self.client.put('/api/me', headers=headers, data=json_data)
            self.assertEquals(response.status_code, 200)
            self.assertEquals(response.json, dict(user=me_to_json(test_user)))
            self.assertEquals(len(outbox), 1)
            self.assertEquals(outbox[0].subject, "Climbing Mont Blanc: Password Changed")

    def test_update_profile_with_wrong_password(self):
        user = User(name="Test", password="password")
        db.session.add(user)
        db.session.commit()

        headers = [('Content-Type', 'application/json')]
        data = {'name': 'Test', 'password': b64encode("password")}
        json_data = json.dumps(data)
        json_data_length = len(json_data)
        headers.append(('Content-Length', json_data_length))
        login_response = self.client.post('/auth/login', headers=headers, data=json_data)
        token = login_response.json['token']

        headers = [('Content-Type', 'application/json')]
        data = {'type': 'email', 'email': 'test@test.no', 'password': b64encode("wrong_password")}
        json_data = json.dumps(data)
        json_data_length = len(json_data)
        headers.append(('Content-Length', json_data_length))
        headers.append(('Authorization', 'Bearer ' + token))

        response = self.client.put('/api/me', headers=headers, data=json_data)
        self.assert401(response)

    def test_update_profile_with_wrong_type(self):
        user = User(name="Test", password="password")
        db.session.add(user)
        db.session.commit()

        headers = [('Content-Type', 'application/json')]
        data = {'name': 'Test', 'password': b64encode("password")}
        json_data = json.dumps(data)
        json_data_length = len(json_data)
        headers.append(('Content-Length', json_data_length))
        login_response = self.client.post('/auth/login', headers=headers, data=json_data)
        token = login_response.json['token']

        headers = [('Content-Type', 'application/json')]
        data = {'type': 'wrong_type', 'email': 'test@test.no', 'password': b64encode("password")}
        json_data = json.dumps(data)
        json_data_length = len(json_data)
        headers.append(('Content-Length', json_data_length))
        headers.append(('Authorization', 'Bearer ' + token))

        response = self.client.put('/api/me', headers=headers, data=json_data)
        self.assert400(response)

    def test_forgot_password(self):
        user = User(name="Test", email="test@test.no", password="password")
        db.session.add(user)
        db.session.commit()

        headers = [('Content-Type', 'application/json')]
        data = {'username': 'Test', 'email': 'test@test.no'}
        json_data = json.dumps(data)
        json_data_length = len(json_data)
        headers.append(('Content-Length', json_data_length))
        with mail.record_messages() as outbox:
            response = self.client.post('/api/forgot_password', headers=headers, data=json_data)
            self.assert200(response)
            self.assertEquals(len(outbox), 1)
            self.assertEquals(outbox[0].subject, "Climbing Mont Blanc: Password reset")

    def test_forgot_password_wrong_mail(self):
        user = User(name="Test", email="test@test.no", password="password")
        db.session.add(user)
        db.session.commit()

        headers = [('Content-Type', 'application/json')]
        data = {'username': 'Test', 'email': 'wrong@test.no'}
        json_data = json.dumps(data)
        json_data_length = len(json_data)
        headers.append(('Content-Length', json_data_length))
        response = self.client.post('/api/forgot_password', headers=headers, data=json_data)
        self.assert400(response)

    def test_forgot_password_wrong_username(self):
        user = User(name="Test", email="test@test.no", password="password")
        db.session.add(user)
        db.session.commit()

        headers = [('Content-Type', 'application/json')]
        data = {'username': 'wrong', 'email': 'test@test.no'}
        json_data = json.dumps(data)
        json_data_length = len(json_data)
        headers.append(('Content-Length', json_data_length))
        response = self.client.post('/api/forgot_password', headers=headers, data=json_data)
        self.assert400(response)
